namespace Grendel.Base {  
   using System;
   using System.Text;
   using System.Globalization;
   using System.Diagnostics;
   using System.Drawing.Drawing2D;
   using System.Drawing;
   using System.Collections.Generic;
   
   /// <summary>
   /// Souřadnice ve 2D prostoru
   /// </summary>
   public struct Position {
      double x;
      double y;
      
        /// <summary>
        /// počátek souřadnic
        /// </summary>
      public static readonly Position Origin = new Position(0.0, 0.0); 
      
      public Position(double x, double y) {
         this.x = x;
         this.y = y;
      }
      
      public double X {
         get {return x;}
         set {x = value;}
      }
      
      public double Y {
         get {return y;}
         set {y = value;}
      }
      
      /// <summary>
      /// vzdálenost od dané pozice k pozici p
      /// </summary>
      public double Distance(Position p) {
         return Math.Sqrt( (X-p.X)*(X-p.X) + (Y-p.Y)*(Y-p.Y) );
      }
      
     
      /// <summary>
      /// test zda je bod uvnitř obdélníku, jenž je určen levým horním a pravým dolním rohem
      /// </summary>
      public bool IsInside(Position leftTop, Position rightBottom) {
         return X >= leftTop.X && X <= rightBottom.X && Y >= leftTop.Y && Y <= rightBottom.Y;
      }
      
      /// <summary>
      /// posunutí pozice ve směru zadaného x a y.
      /// </summary>
      [Obsolete("Use vector operation instead")]
      public Position ShiftBy(double dx, double dy) {
         return new Position(X+dx, Y+dy);
      }
           
      /// <summary>
      /// testování shody dvou bodů (v delta-okolí 1e-6 pixelů)
      /// </summary>
      public bool Equals (Position p) {
         return Math.Abs(X -  p.X) < 1e-6 && Math.Abs(Y - p.Y) < 1e-6;
      }
      
      public override bool Equals (object o)
      {
         if (o is Position)
            return Equals((Position)o);
         else
            return false;
      }

      public static bool operator== (Position p1, Position p2) {
         return p1.Equals(p2);
      }
      
      public static bool operator!= (Position p1, Position p2) {
         return !p1.Equals(p2);
      }      
      
      public override int GetHashCode ()
      {
         return X.GetHashCode() ^ Y.GetHashCode();
      }

      public override string ToString ()
      {
         return string.Format("[{0}, {1}]", X, Y);
      }  
      
       /// <summary>
       ///vektor z pozice origin do target.
       /// </summary>
      public static Vector operator- (Position target, Position origin) {
         return new Vector(target.X - origin.X, target.Y - origin.Y);
      }
      
        /// <summary>
        /// přičtení vektoru k pozici
        /// </summary>
      public static Position operator+ (Position origin, Vector shift) {
         return new Position(origin.X + shift.X, origin.Y + shift.Y);
      }
		
	  public bool IsLeftFrom(Position p) {
			return X < p.X;
	  }
		
	  public bool IsRightFrom(Position p) {
			return X > p.X;
	  }
		
	  public bool IsAbove(Position p) {
			return Y < p.Y;
	  }
		
	  public bool IsBelow(Position p) {
			return Y > p.Y;
	  }
   }
  
    /// <summary>
    /// Vektor ve 2D prostoru
    /// </summary>
  public struct Vector {
      private double x;
      private double y;
      
      public Vector(double x, double y) {
         this.x = x;
         this.y = y;
      }
		
      public double X {
         get {return x;}
         set {x = value;}
      }
      
      public double Y {
         get {return y;}
         set {y = value;}
      }
      
      
        /// <summary>
        /// skalární násobení vektorů
        /// </summary>
      public static double operator* (Vector v1, Vector v2) {
         return v1.X *v2.X + v1.Y * v2.Y;
      }
      
        /// <summary>
        /// násobení vekoru skalárem
        /// </summary>
      public static Vector operator* (double scalar, Vector vector) {
         return new Vector(scalar * vector.X, scalar * vector.Y);
      }
      
       /// <value>
       /// délka vektoru
       /// </value>
      public double Length {
         get {
            return Math.Sqrt(this*this);
         }
      }
      
        /// <summary>
        /// porovnání vektorů (zohledněno je delta-okolí 1e6)
        /// </summary>
      public bool Equals (Vector p)
      {
         return Math.Abs(X - p.X) < 1e-6 
				&& Math.Abs(Y - p.Y) < 1e-6;
      }
      
      public override bool Equals (object o)
      {
         if (o is Vector)
            return Equals((Position)o);
         else
            return false;
      }

      public static bool operator== (Vector p1, Vector p2) {
         return p1.Equals(p2);
      }
      
      public static bool operator!= (Vector p1, Vector p2) {
         return !p1.Equals(p2);
      }      
      
      public override int GetHashCode ()
      {
         return X.GetHashCode() ^ Y.GetHashCode();
      }

      public override string ToString ()
      {
         return string.Format("({0}, {1})", X, Y);
      }
            
   }
  
   /// <summary>
   /// barva v RGB barevném prostoru se specifikací průhůlednosti (alfa-kanál)
   /// </summary>
  public struct Color {
   private byte red;
   private byte green;
   private byte blue;
   private byte transparence;
   
   public Color (byte redComponent, byte greenComponent, byte blueComponent) {
     red = redComponent;
     green = greenComponent;
     blue = blueComponent;
     transparence = 0;
   }  
      
   public Color (byte redComponent, byte greenComponent, byte blueComponent, 
                  byte transparence){
     red = redComponent;
     green = greenComponent;
     blue = blueComponent;
     this.transparence = transparence;
   }       
      
   public Color (int redComponent, int greenComponent, int blueComponent) 
      : this((byte)redComponent, (byte)greenComponent, (byte)blueComponent) {}       

   public Color (int redComponent, int greenComponent, int blueComponent, int transparence) 
      : this((byte)redComponent, (byte)greenComponent, (byte)blueComponent, (byte)transparence)
      {}       
     
     /// <value>
     /// červená složka barvy
     /// </value>
   public byte RedComponent {
      get {return red;}
      set {red = value;}
   }
        
   /// <value>
   /// zelená složka barvy
   /// </value>
   public byte GreenComponent {
      get {return green;}
      set {green = value;}
   }
   /// <value>
   /// modrá složka barvy
   /// </value>  
   public byte BlueComponent {
      get {return blue;}
      set {blue = value;}
   }      

        /// <value>
        /// průhlednost barvy (0 = neprůhledná, 255 - plně průhledná)
        /// </value>
   public byte Transparence {
      get {
         return transparence;
      }
      set {
         transparence = value;
      }
   }
      
   ///<summary>
   /// vrací barvu se změněnou průhledností (ostatní složky jsou přímo zkopírovány)
   ///</summary>
   public Color WithTransparence(byte transparence) {
         Color clone = this;
         clone.Transparence = transparence;
         return clone;
   }
    
   /// <summary>
   /// přimíšení barvy addMixedColor do aktuální barvy. Hodnota quantity určuje
   /// množství přimíšené barvy (hodnota 1.0 vyjadřuje stejné množství přimíšené barvy)
   /// Míšení se provádí v RGB jmeném prostoru. Pro míšení lze použít i operátor + resp. 
   /// * (pro stanovení přimíšeného množství)
   /// </summary>     
   public Color Mix(Color addMixedColor, double quantity) {
      byte nred =  (byte)((red + quantity*addMixedColor.red) / (1+quantity));
      byte ngreen =  (byte)((green + quantity*addMixedColor.green) / (1+quantity));
      byte nblue =  (byte)((blue + quantity*addMixedColor.blue) / (1+quantity));
      byte ntrans =  (byte)((transparence + quantity*addMixedColor.transparence) / (1+quantity));   
      return new Color(nred, ngreen, nblue, ntrans);
    }
   /// <summary>
   /// zkrácená verze metody pro míšení. Neumožňuje stanovit množství
   /// přimíšené barvy (množství je shodné s původní barvou)
   /// </summary>
   public Color Mix(Color addMixedColor) {
      return Mix(addMixedColor, 1.0);
   }
    
   /// <summary>
   /// snadný způsob získání barvy s určeným množstvím (číslo =
   /// první operand), které lze poté použít pro míšení (nejlépe
   /// za pomoci operátoru +)
   /// </summary>   
   public static ColorQuantity operator* (double quantity, Color addmixedColor) {
         return new ColorQuantity(quantity, addmixedColor);
   }
    
   /// <summary>
   /// operátor pro míšení barvy. Odpovídá metodě <see cref="Color.Mix(Color, double)"/>
   /// Druhým operandem je barva se specifikovaným množstvím (získaná
   /// za pomocí operátoru *) 
   /// </summary>
   public static Color operator+ (Color color, Color addMixedColor) {
       return color.Mix(addMixedColor, 1.0);
   }
  
   /// <summary>
   /// operátor pro míšení barvy. Odpovídá metodě <see cref="Color.Mix(Color)"/>
   /// Druhým operandem je přímá barva.  
   /// </summary>   
  public static Color operator+ (Color color, ColorQuantity colorQuantity) {
         return color.Mix(colorQuantity.color, colorQuantity.quantity);
   }      

        /// <summary>
        /// vrací html representaci barvy v RGBA tvaru
        /// </summary>
  public override string ToString ()   {
      return string.Format("#{0:X2}{1:X2}{2:X2}{3:X2}", 
                              RedComponent, GreenComponent, BlueComponent, Transparence);
  } 
    
  /// <summary>
  /// parsuje barvu v HTML RGBA representaci
  /// </summary>
  static public Color Parse(string text) {
      byte red = byte.Parse(text.Substring(1,2), NumberStyles.HexNumber);
      byte green = byte.Parse(text.Substring(3,2), NumberStyles.HexNumber);         
      byte blue = byte.Parse(text.Substring(5,2), NumberStyles.HexNumber);                  
      byte trans = byte.Parse(text.Substring(7,2), NumberStyles.HexNumber);
      return new Color(red, green, blue, trans);         
  }

   public static readonly Color Red  = new Color(255,0,0);
   public static readonly Color Green  = new Color(0,255,0);
   public static readonly Color Blue  = new Color(0,0,255);      
   public static readonly Color White  = new Color(255,255,255);   
   public static readonly Color Black  = new Color(0,0,0);
   public static readonly Color Magenta  = new Color(255,0,255);   
   public static readonly Color Cyan  = new Color(0,255,255);   
   public static readonly Color Yellow  = new Color(255,255,0);   
   public static readonly Color Gray  = new Color(128,128,128);
   public static readonly Color DarkGray  = new Color(64,64,64);      
   public static readonly Color LightGray  = new Color(192,192,192);  
   
   /// <summary>
   ///prázdná barva (bílá plně průhledná). Vhodná pro representaci 
   ///tvarů a jejich částí bez vlastní barvy.
   /// </summary>
   public static readonly Color Empty  = new Color(255, 255, 255, 255);        
  }
   
   /// <summary lang="en">
   /// pomocná struktura spojující barvu s množstvím (vytvářena dočasně při míšení za pomoci operátorů)
   /// </summary>
   public struct ColorQuantity {
      internal double quantity;
      internal Color color;
      
      internal ColorQuantity(double quantity, Color color) {
         this.quantity = quantity;
         this.color = color;
      }
   }
   
  /// <summary>
  /// struktura representují stisknutá tlačítka myši. Podporována jsou
  /// dvě standardní klasická tlačítka a tlačítko prostřední (užívané hlavně v Unixu).  
  /// </summary>
  public struct MouseButtons {  
    bool left;
    bool middle;
    bool right;

    internal MouseButtons(bool left, bool middle, bool right) {
      this.left = left;
      this.right = right;
      this.middle = middle;
    }
     
        /// <value>
        /// test zda bylo stisknuto levé tlačítko
        /// </value>
   public bool IsPressedLeft {
     get { return left;}
   }
      
        /// <value>
        /// test zda bylo stisknuto prostřední tlačítko
        /// </value>
   public bool IsPressedMiddle {
     get { return middle;}
   }   
       
        /// <value>
        /// tets zda bylo stisknuto pravé tlačítko
        /// </value>
   public bool IsPressedRight {
     get { return right;}
   }
      
        /// <value>
        /// test zda bylo stisknuto alespoň jedno (libovolné) tlačítko
        /// </value>
   public bool IsPressedAny {
     get { return IsPressedLeft || IsPressedMiddle || IsPressedRight;}
   }
 }
   
    /// <summary>
    /// výčet standardních stylů písma
    /// </summary>
	public enum FontStyle {
      Normal,
      Italic,
      Bold,
      BoldItalic
   }

    /// <summary>
    /// výčet standardních (generických) řezů písma (konkrétní font závisí na nastevení systému)
    /// </summary>
   public enum GenericFontFamily {
      Unknown,
      Serif,
      SansSerif,
      Monospace
   }

  /// <summary>
  /// přenositelný popis řezu písma
  /// </summary>
  public struct Font {
      private string fontfamily;
      private GenericFontFamily genericFamily;
      private double size;
      private FontStyle style;

       /// <value>
       /// základní styl písma (kurzíva |tučné)
       /// </value>
      public FontStyle Style {
         get {
            return style;
         }
         set {
            style = value;
         }
      }

        /// <value>
        /// stupeň písma 
        /// </value>
      public double Size {
         get {
            return size;
         }
         set {
            size = value;
         }
      }

        /// <value>
        /// generický řez (rodina písma), vlastnost je použitelná jen 
        /// pokud bylo písmo vytvořeno jako generické (pomocí konstruktoru
        /// <see cref="Font.Font(GenericFontFamily, double, FontStyle)"/> apod.).
        /// </value>
      public GenericFontFamily GenericFamily {
         get {
            return genericFamily;
         }
         set {
            genericFamily = value;
         }
      }
      
        /// <value>
        /// řez písma (= jméno fontu). Vlastnost je použitelná pouze pokud
        /// bylo písmo vytvořeno z pojemnovaného fontu (tj. nikoliv generického).
        /// </value>
      public string FontFamily {
         get {
            return fontfamily;
         }
         set {
            fontfamily = value;
         }
      }
    
        /// <value>
        /// vlastnost je pravdivá pokud byl font vytvořen jako generický (tj. s použitím
        /// standardních řezů dané platformy)
        /// </value>
    public bool IsGeneric {
      get {return GenericFamily != GenericFontFamily.Unknown;}
    }
		
        /// <summary>
        /// vytváří generické písmo o velikosti 12 bodů (parametr určuje jaký 
        /// základní druh písma bude použit)
        /// </summary>
		public Font(GenericFontFamily genericFontFamily) 
		: this(genericFontFamily, 12, FontStyle.Normal) {}
		
        /// <summary>
        /// vytváří písmo pojmenovaného fontu (např. Helvetica nebo Arial). Písmo
        /// má velikost 12 bodů a normální styl.
        /// </summary>
		public Font(string fontFamily) 
		: this(fontFamily, 12.0, FontStyle.Normal) {}
		
        /// <summary>
        /// vytváří generické písmo o dané velikosti v daném stylu
        /// </summary>
		public Font(GenericFontFamily genericFontFamily, double size, FontStyle style) {
			 Debug.Assert(genericFontFamily != GenericFontFamily.Unknown, 
                       "Construction of font with unknown fontfamily");
			this.fontfamily = genericFontFamily.ToString();
			this.genericFamily = genericFontFamily;
			this.size = size;
			this.style = style;
		}
		
        /// <summary>
        /// vytváří písmo o dané velikosti v daném stylu (s uvedením přímého jména fontu)
        /// </summary>
		public Font(string fontFamily, double size, FontStyle style) {
			Debug.Assert(fontFamily != "", "Empty fontfamily name");
			this.fontfamily = fontFamily;
			this.genericFamily = GenericFontFamily.Unknown;
			this.size = size;
			this.style = style;			
		}
		
        /// <value>
        /// vrací generické bezpatkové písmo (12 bodů)
        /// </value>
		public static Font SansSerif {
			get {return new Font(GenericFontFamily.SansSerif);}
		}
		
        /// <value>
        /// vrací generické patkové písmo (12 bodů)
        /// </value>        
		public static Font Serif {
			get {return new Font(GenericFontFamily.Serif);}
		}		
		
        /// <value>
        /// vrací generické strojopisné písmo (12 bodů)
        /// </value>         
		public static Font Monospace {
			get {return new Font(GenericFontFamily.Monospace);}
		}	
      
      /// <summary>
      /// textový popis písma (nepřenositelný) 
      /// </summary>
      public override string ToString ()
      {
         if(IsGeneric) {
            return String.Format("#{0}-{1}-{2}", GenericFamily, Style, Size);
         }
         else
           return String.Format("{0}-{1}-{2}", FontFamily, Style, Size); 
      }
  }

    /// <summary>
    /// transformační matice
    /// </summary>
public class WorldMatrix {
	private Matrix matrix;
	
   /// <summary>
   /// vytváří matici identity (tj. matici popisující prázdnou transformaci)
   /// </summary>
	public WorldMatrix() {
		matrix = new Matrix();
	}
		
   /// <summary>
   /// vytváří novou transformační matici klonováním již existující matice
   /// </summary>
	public WorldMatrix(WorldMatrix worldMatrix) {
		matrix = worldMatrix.matrix.Clone();
	}
		
	private WorldMatrix(Matrix m) {
		matrix = m;
	}
		
   /// <summary>
   /// vytváří transformační matici z dvourozměrného pole tranasformačních koeficientů
   /// </summary>
	public WorldMatrix(double [,] m) {
			matrix = new Matrix((float)m[0,0], (float)m[0,1], (float)m[1,0], (float)m[1,1], 
			                    (float)m[2,0], (float)m[2,1]);
	}
	
    /// <summary>
    /// přidá do transformační matici transformaci otáčení o zadaný počet úhlových
    /// stupňů. Mění transformační matici (nevytváří novou matici)
    /// </summary>
	public WorldMatrix Rotate(double angleInDegree) {
		matrix.Rotate((float)angleInDegree, MatrixOrder.Append);
		return this;
	}
	

    /// <summary>
    /// přidá do transformační matici transformaci posunu o zadaný vektor.
    ///  Mění transformační matici (nevytváří novou matici)
    /// </summary>
 	public WorldMatrix Translate(Vector translateVector) {
		matrix.Translate((float)translateVector.X, (float)translateVector.Y, MatrixOrder.Append);
		return this;
	}
        
        
    /// <summary>
    /// přidá do transformační matici transformaci změny měřítka (koeficient lze stanovit pro každou
    /// osu zvlášť)
    ///  Mění transformační matici (nevytváří novou matici)
    /// </summary>        
	public WorldMatrix Scale(double scaleInX, double scaleInY) {
		matrix.Scale((float) scaleInX, (float) scaleInY, MatrixOrder.Append);
		return this;
	}

     /// <summary>
    /// přidá do transformační matici transformaci zkosení podle zadaného vektoru.
    ///  Mění transformační matici (nevytváří novou matici)
    /// </summary>
	public WorldMatrix Shear(Vector shearVector) {
		matrix.Scale((float) shearVector.X, (float) shearVector.Y, MatrixOrder.Append);
		return this;
	}		
	
        
    /// <summary>
    /// přidá do transformační matice další transformace podle předané
    /// matice wm (tj. skládá dvě transformace zadané maticemi, nejdříve
    /// se provedou původní pak přidané). 
    /// Mění transformační matici (nevytváří novou matici)
    /// </summary>
	public WorldMatrix AddTransformations (WorldMatrix wm) {
		matrix.Multiply(wm.matrix, MatrixOrder.Append);
		return this;
	}
	
    /// <value>
    /// vrací inverzní transformační matici tj. matici inverzní (opačné transformace).
    /// Původní transformační matici nemění, vrací vždy novou. 
    /// </value>
	public WorldMatrix InvertedMatrix {
			get {
	    		Matrix im = this.matrix.Clone(); 
				im.Invert();
				return new WorldMatrix(im);
			}
	}
        
    /// <summary>
    /// transformuje bod podle dané (this) transformace
    /// </summary>
	public Position Transform(Position position) {
		PointF[] points = new PointF[]{new PointF((float)position.X, (float)position.Y)};
		matrix.TransformPoints(points);
		return new Position(points[0].X, points[0].Y);
	}

        /// <summary>
        /// převádá transformační matici do dvourozměrného pole koeficientů
        /// (inverzní operace ke konstruktoru <see cref="WorldMatrix(double[,] m)"/>)
        /// </summary>
	public double [,] ToArrayMatrix() {
			return new double[3,3] {
				{matrix.Elements[0], matrix.Elements[1], 0.0},
				{matrix.Elements[2], matrix.Elements[3], 0.0},
				{matrix.Elements[4], matrix.Elements[5], 1.0}
			};
	}
	
	public override string ToString ()
	{
		StringBuilder s = new StringBuilder();
		for(int i=0; i<matrix.Elements.Length; i++) {
			s.AppendFormat("{0:G8} ", matrix.Elements[i]);
			if(i%2 == 1)
				s.Append("\n");
		}
		return s.ToString();
	}

  }
	
/// <summary>
/// Třída pro representaci rovnoběžníků (čtyřúhelníků s rovnoběžnými protilehlými 
/// stranami). Je důležitá neboť rovnoběžníky vznikají transformací obdélníků
/// (resp. jiných rovnoběžníků).
/// </summary>
  public struct Parallelogram {
		Position [] vs;
		
        /// <summary>
        /// tovární metoda pro konstrukci obdélníků (tj. speciálních rovnoběžníků)
        /// z dvou vrcholů (levého horního a pravého dolního)
        /// </summary>
		public static Parallelogram FromSimpleRectangle (Position leftTop, Position rightBottom) {
			return new Parallelogram(leftTop, 
			                          new Vector(rightBottom.X - leftTop.X, 0.0),
			                          new Vector(0.0, rightBottom.Y - leftTop.Y));
		}
		
        /// <summary>
        /// konstrukce rovnoběžníku podle vrcholu a přilehlých vektorů.
        /// </summary>
		public Parallelogram(Position vertex, Vector v1, Vector v2) {
			vs = new Position[4];
			vs[0] = vertex;
			vs[1] = vs[0] + v1;
			vs[2] = vs[1] + v2;
			vs[3] = vs[0] + v2;
		}
		
		private Parallelogram(Position[] nvs) {
			vs = nvs;
		}
		
        /// <summary>
        /// Pozice i-tého vrcholu. Vrcholy jsou uspořádány a to buď
        /// důsledně ve směru hodinových ručiček nebo důsledně proti směru.
        /// </summary>
		public Position Vertex(int i) {
			return vs[i];
		}
		
        /// <value>
        /// testuje zda je rovnoběžník obdélníkem (přesněji pravoúhelníkem)
        /// </value>
		public bool IsRectangle {
			get {
				return Math.Abs((vs[1]-vs[0]) * (vs[2]-vs[1])) 
			             	< double.Epsilon;
			}
		}
		
        /// <summary>
        /// transformuje rovnoběžník na jiný rovnoběžník podle
        /// transformační matice.
        /// </summary>
		public Parallelogram Transform(WorldMatrix m) {
		    Position[] nvs = new Position[4];
			for(int i=0; i<4; i++)
				nvs[i] = m.Transform(vs[i]);
			return new Parallelogram(nvs);
		}
		
        /// <summary>
        /// vrací enumerátor, který umožňuje procházení vrcholů rovnoběžníku.
        /// </summary>
		public IEnumerable<Position> VertexEnumerator() {
			foreach(Position p in vs)
				yield return p;
		}
  }
}