namespace Grendel.Base {  
   using System;
   using System.Collections.Generic;
   using System.Diagnostics;
   using Grendel.Communication;
   using System.IO;
   
   /// <summary>
   /// Základní rozhraní objektů, které kooperují na plátně (= kreslící ploše). Ne všechny objekty plátna 
   /// jsou viditelné.
   /// </summary>
   public interface ICanvasObject : IMessageReceiver {
	  /// <value>
	  /// Plátno, ke kterému objekt přísluší.
	  /// </value>
      Canvas Canvas {get;}
	  /// <summary>
	  /// Přidává objekt do komunikační skupiny v centrální komunikačním uzlu.
	  /// </summary>
	  /// <param name="groupName">
	  /// Identifikace komunikační skupiny.
	  /// </param>
      void AddToCommunicationGroup(string groupName);
	  /// <summary>
	  /// Vyjímá objekt z komunikační skupiny.
	  /// </summary>
      void RemoveFromCommunicationGroup(string groupName);
	  /// <value>
	  /// Kopie seznamu komunikačních skupin, do níž objekt přísluší.
	  /// </value>
      IList<string> ComunnicationGroups {get;}
   }
   
   /// <summary>
   /// Klonovatelné objekty by měly podporovat klonování tj. vytváření kopií s identickými hodnotami všech 
   /// veřejných vlastností a identickým vzhledem (pouze u tvarů). Klon by měl automaticky zaregistrován
   /// do plátna svého vzoru a dědit i účast v komunikačních skupinách. 
   /// </summary>
   public interface ICanvasCloneable {
		/// <summary>
		/// vytvoření klonu. Původní objekt (this) musí zůstat nezměněn.
		/// </summary>
      ICanvasObject Clone(); 
   }
   
   /// <summary>
   /// Rozhraní objektů -- naslouchačů, které zachytávají všechny události myšího kliknutí bez ohledu
   ///  na jeho pozici(tj.  objekt naslouchače nemusí ležet pod kurzorem myši). 
   /// Zachytávající objekt musí být zaregistrován pomocí
   /// metody <see cref="Canvas.AddEventListener(IEventListener)"/> a vlastnost 
   /// <see cref="Canvas.MouseEventTarget"/> musí obsahovat globální příznak EventListeners.
   /// </summary>
   public interface IEventListener {
      void MouseEventHandler(MouseButtons buttons, Position absolutePosition);
   }
   
   /// <summary>
   /// Příznaky určující, která skupina objektů bude informována o myších událostech (= kliknutí). 
   /// Některé příznaky jsou kombinovatelné (například naslouchači s běžnými interaktivními objekty).
   /// </summary>
   ///<remarks>kombinace OneInteractiveShape s InteractiveShapes není definována</remarks>
   [Flags]
   public enum MouseEventTarget {
      /// <summary>
      /// události související s myší jsou zahozeny (musí být uveden jako jediný příznak)
      /// </summary>
      None = 0x0, 
      /// <summary>
      /// myší události jsou doručeny nanejvýše jedinému interaktivnímu tvaru pod kurzorem myši. Volba je
      /// založena nejdříve na vrstvě (preferovány jsou objekty ve vyšších vrstvách), následně 
      /// na vzdálenosti mezi kurzorem a vztažným bodem tvaru (zvolen je nejbližší). Toto nastavení
      /// je počáteční. 
      /// </summary>
      OneInteractiveShape = 0x1,
      /// <summary>
      /// myší událost je doručena všem interaktivním tvarům pod kurzorem
      /// </summary>
      InteractiveShapes = 0x2,
      /// <summary>
      /// myší událost je doručena všem objektům, které se zaregistrovaly jako naslouchači
      /// </summary>
      EventListeners = 0x4
   }
   
   /// <summary>
   /// Kreslící plocha. Aplikace může využívat více pláten, ale každý objekt plátna (např. tvar
   /// může příslušet jen k jedinému plátnu). Pro vytváření pláten použijte 
   /// továrnu <see cref="CanvasFactory"/>
   /// </summary>
   public class Canvas {
      private List<ICanvasObject> objs = new List<ICanvasObject>();
      private SortedList<int, List<IShape> > shapes = new SortedList<int,List<IShape> >();
      private List<IEventListener> listeners = new List<IEventListener>();
      private ICanvasImplementation impl;
      private int height;
      private int width;
      private double animationInterval = 0.0;
      private MouseEventTarget mouseEventTarget = MouseEventTarget.OneInteractiveShape;
      private int frame;
      
      /// <value>
      /// šířka plátna (v pixelech)
      /// </value>
      public int Width {
         get {
            return width;
         }
      }
      
      /// <value>
      /// šířka plátna (v pixelech)
      /// </value>
      public int Height {
         get {
            return height;
         }
      }
      
      /// <value>
      /// animační interval (v sekundách). Nelze jej změnit (počáteční se nastavuje v metodě 
      /// <see cref="Canvas.Display()"/>)
      /// </value>
      public double AnimationInterval {
         get {
            Debug.Assert(animationInterval > 0.0, 
                         "Animation interval is not yet set (before canvas displaying)");
            return animationInterval;
         }
         
      }
      
      internal IDialogBuilder DialogBuilder {
         get {
            return impl.DialogBuilder;
         }
      }

    
      /// <value>
      /// ičíslo aktuálního animačného snímku (je inkrementován od nuly)
      /// </value>
      public int Frame {
         get {
            return frame;
         }
      }

      /// <value>
      /// aktuální nastavení doručování událostí myši
      /// Viz <see cref="T:MouseEventTarget"/>
      /// </value>
      public MouseEventTarget MouseEventTarget {
         get {
            return mouseEventTarget;
         }
         set {
            mouseEventTarget = value;
         }
      }
      
      internal Canvas(int width, int height, ICanvasImplementation implementation) {
         impl = implementation;
         implementation.Repainting += Repainting;
         this.width = width;
         this.height = height;
		 Trace.Listeners.Add(new TextWriterTraceListener(Console.Error));
      }
      
      /// <summary>
      /// Registrace objektu plátna. Každý objekt plátna musí být registrován (tj.
      /// musí volat resp. musí být pro něj volána tato metoda). Typicky se tak děje v konstruktoru.
      /// </summary>

      public void Remember(ICanvasObject newCanvasObject) {
         objs.Add(newCanvasObject);
         if(newCanvasObject is IShape) {
            IShape shape = (IShape)newCanvasObject;
            int layer = -shape.Layer;
            if(!shapes.ContainsKey(layer))
               shapes[layer] = new List<IShape>();
            shapes[layer].Add(shape);
         }
         impl.Invalidate();
      }
      
      /// <summary>
      /// Odregistrování objektu plátna (objekt je parametrem metody). 
      /// Objekt musí být přirozeně předtím zaregistrován (viz
      /// <see cref="Canvas.Remember"/>).
      /// </summary>
      public void Forget(ICanvasObject fadingObject) {
         if(!objs.Contains(fadingObject))
            throw new ArgumentException("Fading object is not remembered by canvas");
         objs.Remove(fadingObject);
         if(fadingObject is IShape) {
            IShape shape = (IShape)fadingObject;
            int layer = shape.Layer;  
            shapes[layer].Remove(shape);
         }
		 Trace.Write(string.Format("Object of class {0} have been forgotten", fadingObject.GetType().Name));	
         impl.Invalidate();
      }
      
      /// <summary>
      /// Zobrazuje plátno v okně (pokud již neexistuje) a startuje animaci. Zbytek aplikace
      /// je prováděn v rámci událostní smyčky, tj. volání této metody <strong>nikdy nekončí</strong>
      /// Veškeré další aktivity programu musí být vykonávány jako reakce na myší a animační události.
      /// </summary>
      /// <param name="animationInterval"> animační interval ve vteřinách. Hodnoty pod 0.1 sekundy
      /// nejsou příliš využitelné (doba překreslení může být vyšší). Animaci lze potlačit použitím
      /// extrémně dlouhého animačního intervalu (např. double.MaxValue).
      /// </param>
      public void Display(double animationInterval) {
         //the application is blocked in this call!!
         this.animationInterval = animationInterval;
         impl.MouseInteraction += MouseEventDispatcher;
         impl.Animation += Animation;
         frame = 0;
         impl.Display(width, height,(int)(animationInterval*1000));
      }
      
      /// <summary>
      ///  /// Zobrazuje plátno v okně (pokud již neexistuje) a startuje animaci
      /// s animačním intervalem 0,1 vteřiny. Zbytek aplikace
      /// je prováděn v rámci událostní smyčky, tj. volání této metody <strong>nikdy nekončí</strong>
      /// Veškeré další aktivity programu musí být vykonávány jako reakce na myší a animační události.
      /// </summary>
      public void Display() {
         Display(0.1);
      } 
      
      protected virtual void Repainting() {
         foreach(int layer in shapes.Keys) {
           foreach(IShape shape in shapes[layer])
               shape.Painting();
         }
      }
      
	  /// <summary>
	  /// Načítá bitmapu ze datového proudu (tj. například ze souboru). Podporované formáty
	  /// závisí na použité knihovně, podpora formátů JPG resp. PNG je však všeobecná.
	  /// </summary>
      public IBitmap LoadBitmap(Stream bitmapStream) {
         return impl.LoadBitmap(bitmapStream);
      }
      
      internal void Invalidate() {
         impl.Invalidate();
      }
      
          
      internal void MoveToLayer(IShape shape, int oldLayer, int newLayer) {
         shapes[oldLayer].Remove(shape);
         if(!shapes.ContainsKey(newLayer))
            shapes[newLayer] = new List<IShape>();
         shapes[newLayer].Add(shape);
		 Invalidate();
      }

      /// <summary>
      /// Výška textu zobrazeného příslušným řezem písma v pixelech.
      /// </summary>
      public double TextHeight(string text, Font font) {
            return impl.TextHeight(font, text);
      }
      
      /// <summary>
      /// Šířka textu zobrazeného příslušným řezem písma v pixelech.
      /// </summary> 
      public double TextWidth(string text, Font f) {
            return impl.TextWidth(f, text);
      }
      
      /// <summary>
      /// Kreslí na plátno úsečku z bodu startPointLocation do bodu endPointPointLocation. Úsečka 
      /// musí být kreslena v rámci metody <see cref="IShape.Painting()"/> určitého tvaru. Tento
      /// tvar musí být předán jako první parametr. Pozice jsou relativní ke vztažnému bodu tvaru.
      /// </summary>
      public void DrawLine(IShape shape, Position startPointLocation, 
                            Position endPointLocation, double lineWidth, Color color) 
      {
		 WorldMatrix m = shape.TotalTransformation;
         Position p1 = startPointLocation;
         Position p2 = endPointLocation;
         impl.DrawLine(m, (float)p1.X, (float)p1.Y, (float)p2.X, (float)p2.Y, (float)lineWidth, color); 
      }
     
     /// <summary>
     /// Zkrácená verze volání <see cref="Canvas.DrawLine(IShape, Position, Position, int, Color)"/>
     /// s šířkou rovnou jednomu pixelu.
     /// </summary> 
     public  void DrawLine(IShape shape, Position startPointLocation,
                            Position endPointLocation, Color color) {
         DrawLine(shape, startPointLocation, endPointLocation, 1, color);
      }
      
      /// <summary>
      /// Kreslí na plátno kružnici ze středem na pozici centerLocation a poloměrem r. Kružnice
      /// musí být kreslena v rámci metody <see cref="IShape.Painting()"/> určitého tvaru. Tento
      /// tvar musí být předán jako první parametr. Pozice jsou relativní ke vztažnému bodu tvaru.
      /// </summary>     
      public void DrawCircle(IShape shape, Position centerLocation, double r, double lineWidth, Color color) {
		 WorldMatrix m = shape.TotalTransformation;
         Position c = centerLocation;
         impl.DrawCircle(m, (float)c.X, (float)c.Y, (float)r, (float)lineWidth, color);
      }
      
    /// <summary>
      /// <summary>
      /// Kreslí na plátno text daným fontem a danou barvou. Text
      /// musí být kreslen v rámci metody <see cref="IShape.Painting()"/> určitého tvaru. Tento
      /// tvar musí být předán jako první parametr. Pozice jsou relativní ke vztažnému bodu tvaru.
      /// </summary>  
    /// </summary>
    public void DrawText(IShape shape, Position topLeftLocation, string text, 
                          Font font, Color color)	{
		 WorldMatrix m = shape.TotalTransformation;
         Position p = topLeftLocation;
         impl.DrawText(m, (float)p.X, (float)p.Y, text, color, font);
    }

    /// <summary>

    /// Mírně zjednodušená verze metody
    /// <see cref="Canvas.DrawText(IShape, Position, string, Font, Color)"/>. Kreslí text
    /// strojopisem a černou barvou.
    /// </summary>

     public void DrawText(IShape shape, Position topLeftLocation, string text)	{
         DrawText(shape, topLeftLocation, text, Font.SansSerif, Color.Black);
     }
     
      /// <summary>
	  /// Kreslí obdélník mezi určenými pozicemi. Na rozdíl od kresby pomocí
	  /// čtyř úseček zajišťuje hladké navázání stran. Obdélník
      /// musí být kreslen v rámci metody <see cref="IShape.Painting()"/> určitého tvaru. Tento
      /// tvar musí být předán jako první parametr. Pozice jsou relativní ke vztažnému bodu tvaru.
      /// </summary>
     public void DrawRectangle(IShape shape, Position topLeftLocation, 
                                 Position bottomRightLocation, double width, Color color) {
		 WorldMatrix m = shape.TotalTransformation;
         Position l = topLeftLocation;
         Position r = bottomRightLocation;
         impl.DrawLine(m, (float)l.X, (float)l.Y, (float)r.X, (float)l.Y, (float) width,  color);
         impl.DrawLine(m, (float)l.X, (float)l.Y, (float)l.X, (float)r.Y, (float) width, color);
         impl.DrawLine(m, (float)r.X, (float)r.Y, (float)l.X, (float)r.Y, (float) width, color);
         impl.DrawLine(m, (float)r.X, (float)r.Y, (float)r.X, (float)l.Y, (float) width, color);
      }
      
	 /// <summary>
	 /// Zkrácená verze metody 
	 /// <see cref="Canvas.DrawRectangle(IShape, Position, Position, double, Color)"/>
	 /// s šířkou linek jeden pixel.
	 /// </summary>
      public void DrawRectangle(IShape shape, Position topLeftLocation, 
                                 Position bottomRigthLocation, Color color) {
         DrawRectangle(shape, topLeftLocation, bottomRigthLocation, 1.0, color);
      }

	 /// <summary>
	 /// Kreslí vyplněný trojúhelník (vrcholy zadány jednotlivě). Trojúhelník
      /// musí být kreslen v rámci metody <see cref="IShape.Painting()"/> určitého tvaru. Tento
      /// tvar musí být předán jako první parametr. Pozice jsou relativní ke vztažnému bodu tvaru. 
	 /// </summary>
     public void DrawFilledTriangle(IShape shape, Position p1Location, 
                                     Position p2Location, Position p3Location, Color color) {
		 WorldMatrix m = shape.TotalTransformation;
         Position p1 = p1Location;
         Position p2 = p2Location;			
		 Position p3 = p3Location;					 	
         impl.FillTriangle(m, (float)p1.X, (float)p1.Y, (float)p2.X, (float)p2.Y, 
                           (float)p3.X, (float)p3.Y, color, true);
      }
 
	 /// <summary>
	 /// Kreslí vyplněný trojúhelník (vrcholy zadány v poli). Trojúhelník
     /// musí být kreslen v rámci metody <see cref="IShape.Painting()"/> určitého tvaru. Tento
     /// tvar musí být předán jako první parametr. Pozice jsou relativní ke vztažnému bodu tvaru. 
	 /// </summary>		
    public void DrawFilledTriangle(IShape shape, Position[] cornerLocations, Color color) {
         Debug.Assert(cornerLocations.Length == 3, "Triangle must contains 3 corners");
         DrawFilledTriangle(shape, cornerLocations[0], cornerLocations[1], 
                              cornerLocations[3], color);
      }
    
	/// <summary>
	/// Kreslí bitmapu. Objekt bitmapy lze získat voláním metody <see cref="Canvas.LoadBitmap(Stream)"/>.
	/// Bitmapa
    /// musí být kreslena v rámci metody <see cref="IShape.Painting()"/> určitého tvaru. Tento
    /// tvar musí být předán jako první parametr. Pozice jsou relativní ke vztažnému bodu tvaru.
	/// </summary>
    public void DrawBitmap(IShape shape, Position topLeftLocation, IBitmap bitmap) {
	  WorldMatrix m = shape.TotalTransformation;
      Position p = topLeftLocation;					
      impl.DrawBitmap(m, (float)p.X, (float)p.Y, bitmap);
    }
	
		
	/// <summary>
	/// Kreslí beziérovou kubiku (křivku) určenou čtyřmi body.  Křivka
    /// musí být kreslena v rámci metody <see cref="IShape.Painting()"/> určitého tvaru. Tento
    /// tvar musí být předán jako první parametr. Pozice jsou relativní ke vztažnému bodu tvaru.
	/// </summary>	
	public void DrawBezierCurve(IShape shape, Position p0, Position p1, Position p2,
		                         Position p3, double widthOfCurve, Color color) {
	  	WorldMatrix m = shape.TotalTransformation;	
		impl.DrawBezier(m, (float)p0.X, (float)p0.Y,
			               (float)p1.X, (float)p1.Y, 
			               (float)p2.X, (float)p2.Y, 
			               (float)p3.X, (float)p3.Y, (float)widthOfCurve, color);
		}

	/// <value>
	/// vrací iterátor určený pro procházení všech objektů na plátně. Procházení se děje přes kopii
	/// seznamu (nemůže nastat kolize, ale výčet nemusí být aktuální).
	/// </value>
	public IEnumerator<ICanvasObject> CanvasObjects {
			get {
				return (new List<ICanvasObject>(objs)).GetEnumerator();
			}
	}

	/// <value>
	/// vrací iterátor určený pro procházení všech objektů určitého (pod)typu T na plátně. 
	/// Procházení se děje přes kopii seznamu (nemůže nastat kolize, ale výčet nemusí být aktuální).
	/// </value>
	public IEnumerator<T> ObjectsOfType<T>() where T : ICanvasObject {
		foreach(ICanvasObject obj in new List<ICanvasObject>(objs))
					if(obj is T)
						yield return (T)obj;		
	}

	/// <value>
	/// vrací iterátor určený pro procházení všech viditelných tvarů na plátně. 
	/// Procházení se děje přes kopii seznamu (nemůže nastat kolize, ale výčet nemusí být aktuální).
	/// </value>
	public IEnumerator<IShape> Shapes {
			get {
				return ObjectsOfType<IShape>();							
			}
	}
      
    private delegate void InteractionDelegate();
      
    private List<InteractionDelegate> FindOptimalShape(MouseButtons buttons, 
                                                      Position absolutePosition) {
         List<InteractionDelegate> result = new List<InteractionDelegate>();
         double min = double.MaxValue;
         IInteractiveShape optTarget = null;
         Position optClickLocation = Position.Origin;
         foreach(int layer in shapes.Keys) {
            foreach(IShape shape in shapes[layer]) {
               if(shape is IInteractiveShape) {
                  IInteractiveShape target = (IInteractiveShape) shape;
				  if (!target.IsActive)
							continue;
                  Position clickLocation = 
                     target.TotalTransformation.InvertedMatrix.Transform(absolutePosition);
                  if(clickLocation.IsInside(target.LeftTopCorner, target.RightBottomCorner)) {
                     double dist = Position.Origin.Distance(clickLocation);
                     if(dist < min) {
                        min = dist;
                        optTarget = target;
                        optClickLocation = clickLocation;
                     }  
                  }
               }
            }
            if(optTarget != null) {
               result.Add( delegate () {optTarget.Interaction(buttons, optClickLocation);});
               return result;
            }
         }
         return result;
    }
      
    private List<InteractionDelegate> FindShapes(MouseButtons buttons, Position absolutePosition) {
         List<InteractionDelegate> result = new List<InteractionDelegate>();
         foreach(int layer in shapes.Keys) {
            foreach(IShape shape in shapes[layer]) {
               if(shape is IInteractiveShape) {
                  IInteractiveShape target = (IInteractiveShape) shape;
				  if (!target.IsActive)
						continue;
                  Position clickLocation = 
  						target.TotalTransformation.InvertedMatrix.Transform(absolutePosition);                   
                  if(clickLocation.IsInside(target.LeftTopCorner, target.RightBottomCorner)) {
                     result.Add(delegate(){target.Interaction(buttons, clickLocation);});
                  }
               }
            }
         }
         return result;
    }
      
    protected virtual void MouseEventDispatcher(MouseButtons buttons, 
                                                 Position absolutePosition) {
         MouseEventTarget savedTarget = mouseEventTarget;
         List<InteractionDelegate> interactions = new List<InteractionDelegate>();
         if((savedTarget & MouseEventTarget.OneInteractiveShape) != 0)
            interactions.AddRange(FindOptimalShape(buttons, absolutePosition));
         if((savedTarget & MouseEventTarget.InteractiveShapes) != 0)
            interactions.AddRange(FindShapes(buttons, absolutePosition));
         //dispatch to interactive shape
         foreach(InteractionDelegate interaction in interactions) {
            interaction();
         }
         //dispatch to listener
         if((savedTarget & MouseEventTarget.EventListeners) != 0) {
            foreach(IEventListener listener in listeners.ToArray()) {
                     listener.MouseEventHandler(buttons, absolutePosition);
            }
         }
    }
      
      protected virtual void Animation() {
         bool repaint = false;
         
        foreach(ICanvasObject o in objs.ToArray()){ //iteration on copy of list (list is muttable)
           if(o is IDynamicObject)
                  repaint = ((IDynamicObject)o).AnimationStep(frame) || repaint;
         }
         if(repaint)
            impl.Invalidate();
         frame++;
      }
      
      /// <summary>
      /// registruje naslouchače myších událostí (<see cref="Canvas.MouseEventTarget"/>)
      /// </summary>
      public void AddEventListener(IEventListener listener) {
         listeners.Add(listener);
      }
      
      /// <summary>
      /// odregistruje naslouchače myších událostí (<see cref="Canvas.MouseEventTarget"/>)
      /// </summary>
      /// <param name="listener">
      /// [<see cref="IEventListener"/>] listener, which does not already want to receive events.
      /// </param>     
      public void RemoveEventListener(IEventListener listener) {
         listeners.Remove(listener);
      }
   }
   
   /// <summary>
   /// Továrna pro vytváření kreslících ploch (pláten) s využitím zvolené kreslící technologie.
   /// </summary>
   public class CanvasFactory {
      private int width;
      private int height;
		
	  private static Dictionary<string, ICanvasImplementation> imap;
		
	  static CanvasFactory() {
			imap = new Dictionary<string, ICanvasImplementation>();
          	imap["WinForms"] = new WFormsCanvasImplementation();			
	  }
   
   /// <summary xml:lang="en">
   /// vytvoření továrny, která poskytuje plátna s daným rozměrem.
   /// </summary>
      public CanvasFactory(int width, int height) {
         Debug.Assert(width >= 100 && width <= 10000);
         Debug.Assert(height >= 100 && height <= 10000);         
         this.width = width;
         this.height = height;
      }      

      /// <summary>
      /// vytvoření továrny na plátna s rozměrem 500 x 500
      /// </summary>
      public CanvasFactory() : this(500, 500) {}
      
      /// <value>
 	  /// vytvoření plátna s využitím optimální dostupné a použitelné knihovny   
      /// </value>
      public Canvas BestCanvas {
            get {return CanvasByFramework("WinForms");}
      }
       
      /// <value>
      /// vytvoření plátna s využitím přenositelné knihovny (dostupné na všech .NET platformách)
      /// </value>
      public Canvas StandardCanvas {
            get {return CanvasByFramework("WinForms");}         
      }
      
      /// <summary>
	  /// vytvoření plátna využívajícího specifikovanou kreslící technologii. Identifikátory dostupných
	  /// technologií lze získat voláním statické vlastnosti <see cref="Canvas.AvailableFrameworks"/>
      /// </summary>
      public Canvas CanvasByFramework(string frameworkName) {
          if (!imap.ContainsKey(frameworkName)) {
            string[] keys = new string[imap.Count];
            imap.Keys.CopyTo(keys, 0);
            string supported = string.Join(",", keys);
            throw new NotSupportedException(
                 string.Format("This framework is not supported. Supported frameworks: {0}", 
                               supported)); 
          }
          return new Canvas(width, height, imap[frameworkName]);
      }
		
	 /// <value>
	 /// iterátor přes všechny dostupné kreslící knihovny (technologie rep. rámce) 
	 /// </value>	
	 public static IEnumerator<string> AvailableFrameworks {
		get {return imap.Keys.GetEnumerator();}	
	 }
	
	/// <summary>
	/// Adaptér umožňující použít jako kreslící plochu libovolný widget, který podporuje 
	/// kreslení. Nabídka podporovaných widgetl 
	/// je dána jednotlivými podporovanými kreslícími knihovnami a jejich podporou v Grendelu 
	/// (minimálně je to cokoliv odvozené z System.Windows.Forms.Control).
	/// </summary>

	 public static Canvas Adapter(object existingCanvas, double animationInterval) {
			if (existingCanvas is System.Windows.Forms.Control) {
				System.Windows.Forms.Control control = existingCanvas as System.Windows.Forms.Control;
				return new Canvas(control.Width, control.Height, 
				                   new WFormsCanvasImplementation(control, (int)(animationInterval * 1000)));
			}
			else
				throw new ArgumentException(string.Format("Adapter does not exist for class {0}", existingCanvas.GetType().FullName));
	  }
   }
   
	
}
