namespace Grendel.Base {
   using System;
   using System.Drawing;
   using System.Drawing.Text;
   using System.Drawing.Drawing2D;  
   using System.Windows.Forms;
   using System.Diagnostics;  
   using System.Collections.Generic;
   using System.Reflection;
   using System.IO;
   
   public delegate void AnimationEvent();
   public delegate void PaintEvent();
   public  delegate void MouseEvent(MouseButtons buttons, Position absolutePosition);
   
    /// <summary>
    /// rozhraní přenositelného kreslení na plátno. Z důvodů snadné přenositelnosti
    /// je minimalizované a pro přenášení pozic užívá jednoduchých parametrů typu
    /// float (kreslící knihovna by měla podporovat float souřadnice). 
    /// </summary>
   public interface ICanvasImplementation {
      void DrawLine(WorldMatrix m, float x1, float y1, float x2, float y2, float width, Color color);
      void FillTriangle(WorldMatrix m, float x1, float y1, float x2, float y2, float x3, float y3, 
                        Color color, bool margin);
      void DrawText(WorldMatrix m, float x, float y, string text, Color color, Font font);
      void Display(int width, int height, int msecAnimationInterval);
      void DrawBitmap(WorldMatrix m, float x, float y, IBitmap bitmap);
      void DrawCircle(WorldMatrix m, float x, float y, float r, float lineWidth, Color color);
	  void DrawBezier(WorldMatrix m, float p0x, float p0y, float p1x, float p1y,
		                              float p2x, float p2y, float p3x, float p3y,
		                              float width, Color color);
      double TextWidth(Font f, string text);
      double TextHeight(Font f, string text);
      /// <value>
      /// poskytuje objekt pro přenositelné vytváření dialogů vlastností
      /// </value>
      IDialogBuilder DialogBuilder {get;}
      IBitmap LoadBitmap(Stream bitmapStream);
      void Invalidate();
        /// <summary>
        /// událost vyvolávaná při požadavku na překreslení (včetně požadavku po
        /// volání metody <see cref="ICanvasImplementation.Invalidate"/>.
        /// </summary>
      event PaintEvent Repainting;
        /// <summary>
        /// událost vyvolávaná časovačem (timerem) v intervalu animací
        /// </summary>
      event AnimationEvent Animation;
        /// <summary>
        /// událost vyvolávaná při stisku myši
        /// </summary>        
      event MouseEvent MouseInteraction;
   }
   
   [Flags]
   public enum StandardButtons {
      OK = 0x1,
      Cancel = 0x2,
      None = 0
   }
   
    /// <summary>
    /// Rozhraní poskytovatele jednoduchých dialogů. Jednotlivé aktivní prvky jsou
    /// určeny jedinečnými textovými klíči, které zároveň slouží jako popisky.
    /// </summary>
   public interface IDialogBuilder {
      /// <summary>
      /// přidání tlačítka s daným jednoznačným klíčem. Volá se před zobrazením dialogového
      /// okna.
      /// </summary> 
      void AddButton(string key);
        /// <summary>
        /// přidání vstupního pole (s popiskem). Pole je identifikováno klíčem. Volá se před zobrazením dialogového
        /// okna.
        /// </summary>
      void AddInput(string key, object initialValue);
        /// <summary>
        /// vložení vizuálního oddělovače (typicky vodorovná linka). Volá se před zobrazením dialogového
        /// okna.
        /// </summary>
      void AddSeparator();
        /// <summary>
        /// zobrazení dialogu. Je možno specifikovat přidání standardních tlačítek.
        /// Po stisku libovolného tlačítka je dialogové okno uzavřeno a vrácen
        /// je klíč tlačítka. U standardních tlačítek je klíč "OK" resp. "Cancel".
        /// </summary>
      string ExecuteDialog(int minimalWidth, StandardButtons stdButtons);
        /// <summary>
        /// metoda umožňuje po uzavření dialogu získat hodnotu vstupního pole (pole je určeno
        /// svým klíčem).
        /// </summary>
      object GetValue(string key);
       /// <value>
       /// vrací iterátor přes klíče všech tlačítek a vstupních polí.
       /// </value>
      IEnumerable<string> Keys {get;}
   }
   
   public interface IBitmap {
      void Dispose();
      int Width {get;}
      int Height {get;}
   }
   
    /// <summary>
    /// implementace plátna pomocí Windows Forms (verze 1.0).
    /// </summary>
   public class WFormsCanvasImplementation : ICanvasImplementation {
      private Control form = null; 
      private Timer timer = new Timer();
      private Graphics painter = null;
      
      public event PaintEvent Repainting;      
      public event AnimationEvent Animation;
      public event MouseEvent MouseInteraction;
		
	  public WFormsCanvasImplementation() {}
   
	  internal WFormsCanvasImplementation(Control c, int msecAnimationInterval) {
			form = c;
       		form.Paint += this.OnPaint;
         	form.MouseDown += this.OnMouseDown;
			timer.Tick += this.OnTick;
         	timer.Interval = msecAnimationInterval;
         	timer.Start();
	  }
        
      public void Display (int width, int height, int msecAnimationInterval) {
		 if (form != null)
				return;
         form = new DBForm();
         form.Width = width;
         form.Height = height;
         form.BackColor = System.Drawing.Color.White;
         form.Paint += this.OnPaint;
         form.MouseDown += this.OnMouseDown; 
         form.Text = "Drawing area";
         timer.Tick += this.OnTick;
         timer.Interval = msecAnimationInterval;
         timer.Start();
         Application.Run((Form)form);
      }
 
      internal System.Drawing.Color ToDrawingColor(Color c) {
         return System.Drawing.Color.FromArgb(255 - c.Transparence, c.RedComponent, 
                               c.GreenComponent, c.BlueComponent);
      }
		
	  internal void Transform(WorldMatrix wm, Graphics g) {
			double [,] am = wm.ToArrayMatrix();
			Matrix m = new Matrix((float)am[0,0], (float)am[0,1], (float)am[1,0], (float)am[1,1], 
			                      (float)am[2,0], (float)am[2,1]);
			g.Transform = m;
	  }

      internal System.Drawing.Font ToDrawingFont(Font f) {
         System.Drawing.FontStyle style;
         if (f.Style == FontStyle.Italic || f.Style == FontStyle.BoldItalic)
            style = System.Drawing.FontStyle.Italic; 
         else 
            style = System.Drawing.FontStyle.Regular;
         if (f.Style == FontStyle.Bold || f.Style == FontStyle.BoldItalic)
            style |= System.Drawing.FontStyle.Bold; 
      
         FontFamily family = null;
         if(f.IsGeneric)
            switch(f.GenericFamily) {
            case GenericFontFamily.Serif:
                  family = new FontFamily(GenericFontFamilies.Serif);
                  break;
            case GenericFontFamily.SansSerif:
                  family = new FontFamily(GenericFontFamilies.SansSerif);
                  break;        
            case GenericFontFamily.Monospace:
                  family = new FontFamily(GenericFontFamilies.Monospace);
                  break; 
            default:
                  Debug.Fail("Unknown generic fontfamily");
                  break;
            }
         else
            family = new FontFamily(f.FontFamily);
         return new System.Drawing.Font(family, (float)f.Size, style, GraphicsUnit.Pixel);
      }
      
      public void DrawLine (WorldMatrix m, float x1, float y1, float x2, float y2, float width, Color color) {
         if(painter != null) {
			Transform(m, painter);
            painter.DrawLine(new Pen(ToDrawingColor(color), width), x1, y1, x2, y2);  
		 }
      }
		
	  public void DrawBezier(WorldMatrix m, float p0x, float p0y, float p1x, float p1y,
		                              float p2x, float p2y, float p3x, float p3y,
		                              float width, Color color) {
			if(painter != null) {
				Transform(m, painter);
				painter.DrawBezier(new Pen(ToDrawingColor(color), width), p0x, p0y, p1x, p1y,
				                   			p2x, p2y, p3x, p3y);
			}
	  }
      
      public void DrawCircle (WorldMatrix m, float x, float y, float r, float lineWidth, Color color)
      {
        if(painter != null) {
			Transform(m, painter);				
            painter.DrawEllipse(new Pen(ToDrawingColor(color), lineWidth), x, y, 2*r, 2*r); 
		}
      }

      
      public void DrawText (WorldMatrix m, float x, float y, string text, Color color, Font font) {	
         if(painter != null) {
	     	Brush b = new SolidBrush(ToDrawingColor(color));
         	System.Drawing.Font f = ToDrawingFont(font);
   			Transform(m, painter);	      				
            painter.DrawString(text, f, b, (float)x, (float) y);
		 }
      }
      
      public void FillTriangle (WorldMatrix m, float x1, float y1, float x2, float y2, float x3, float y3,
                                Color color, bool margin) {
         if(painter != null) {
			Brush b = new SolidBrush(ToDrawingColor(color));
         	PointF[] pts = new PointF[]{new PointF(x1, y1), new PointF(x2, y2), new PointF(x3, y3)};
		    Transform(m, painter);	
            painter.FillPolygon(b, pts);
            if(margin)
               painter.DrawPolygon(new Pen(new SolidBrush(ToDrawingColor(color)), 0.5f), pts);
         }
         
      }
      
      public void DrawBitmap (WorldMatrix m, float x, float y, IBitmap bitmap) {
         if (painter == null)
            return;
		 Transform(m, painter);	
         painter.DrawImage(((WFBitmap)bitmap).InternalBitmap, x, y);
      }

      
      public IBitmap LoadBitmap (Stream bitmapStream) {
         return WFBitmap.LoadFromFile(bitmapStream);
      }

      
      public void Invalidate() {
         if(form != null)
            form.Invalidate();
      }

      
      private void OnPaint(object sender, PaintEventArgs e) {
         painter = e.Graphics;
         painter.SmoothingMode = SmoothingMode.AntiAlias;
         Repainting();
         painter = null;  
      }
  
      private void OnMouseDown(object sender, MouseEventArgs e) {
         MouseInteraction(
             new MouseButtons(e.Button == System.Windows.Forms.MouseButtons.Left, 
                               e.Button == System.Windows.Forms.MouseButtons.Middle, 
                               e.Button == System.Windows.Forms.MouseButtons.Right
                               ),
             new Position(e.X, e.Y)             
                         );
      }
      
      private void OnTick(object sender, EventArgs e) {
         if(Animation != null)
            Animation();
      }
      
      public IDialogBuilder DialogBuilder {
         get { return new WFormDialogBuilder(); }
      }
        
     
      public double TextHeight (Font f, string text) {
        using (Form pf = new Form()) {
            using (Graphics p = pf.CreateGraphics()) {
                return p.MeasureString(text, ToDrawingFont(f)).Height;
            }
        }
      }

    public double TextWidth (Font f, string text) {
        using (Form pf = new Form()) {
            using (Graphics p = pf.CreateGraphics()) {
                return p.MeasureString(text, ToDrawingFont(f)).Width;
            }
        }
      }
   }
   
   internal class WFBitmap : IBitmap {
      private Bitmap ib = null;
      
      private WFBitmap() {}
      
      public static IBitmap LoadFromFile (Stream bitmapStream) {
         WFBitmap b = new WFBitmap();
         b.ib = new Bitmap(bitmapStream);
         return b;
      }
      
      public void Dispose ()
      {
         ib.Dispose();
         ib = null;
      }

      public int Height {
         get { 
            Debug.Assert(ib != null, "Undefined bitmap");
            return ib.Height; }
      }

      public int Width {
         get {
            Debug.Assert(ib != null, "Undefined bitmap");
            return ib.Width;
         }
      }
      
      internal Bitmap InternalBitmap {
         get {
            Debug.Assert(ib != null, "Undefined bitmap");
            return ib;
         }
      }

   }
   
   internal class WFormDialogBuilder : IDialogBuilder {
      private Dictionary<string, Control> keyToControl = new Dictionary<string,Control>();
      private Dictionary<Control, string> controlToKey = new Dictionary<Control,string>();
      private Dictionary<string, object> initialValues = new Dictionary<string,object>();
      private List<string> keys = new List<string>();
      Form dialog = null;
      string clickedButton;
      
      public void AddButton (string key) {
         Button b = new Button();
         b.Text = key;
         b.Tag = true;
         keyToControl[key] = b;
         controlToKey[b] = key;
         keys.Add(key);
      }
      
      public void AddInput (string key, object initialValue) {
         keys.Add(key);
         initialValues[key] = initialValue;
         if(initialValue.GetType() == typeof(bool)) {
            ComboBox cb = new ComboBox();
            cb.Items.AddRange(new string[]{"true", "false"});
            cb.SelectedIndex = (bool)initialValue ? 0 : 1;
            cb.DropDownStyle = ComboBoxStyle.DropDownList;
            keyToControl[key] = cb;
            controlToKey[cb] = key;
            cb.Tag = false;
         }
         else if (initialValue.GetType() == typeof(Color)) {
				Color c = (Color) initialValue;
				Button b = new Button();
				b.Text = initialValue.ToString();
				b.BackColor = System.Drawing.Color.FromArgb(255 - c.Transparence, c.RedComponent, 
                               c.GreenComponent, c.BlueComponent);
		        b.Click += delegate (Object sender, EventArgs e) {
					ColorDialog d = new ColorDialog();
					d.Color = b.BackColor;
					if (d.ShowDialog() == DialogResult.OK) {
						System.Drawing.Color nc = d.Color;
						b.BackColor = nc;
						b.Text = (new Color(nc.R, nc.G, nc.B, 255 - nc.A)).ToString();
					}
			};
			keyToControl[key] = b;
            controlToKey[b] = key;
            b.Tag = false;
		 }
         else {
            TextBox tb = new TextBox();
            tb.Text = initialValue.ToString();
            keyToControl[key] = tb;
            controlToKey[tb] = key;
            tb.Tag = false;
         }
      }
      
      public void AddSeparator ()
      {
         throw new NotImplementedException ();
      }

      
      public string ExecuteDialog (int minimalWidth, StandardButtons stdButtons)
      {
         int count = keys.Count;
         dialog = new Form();
         dialog.Width = minimalWidth;
         dialog.Height = (count + (stdButtons != StandardButtons.None ? 2 : 1)) * 32;
         for(int i=0; i<count;i++) {
            string key = keys[i];
            Control c = keyToControl[key];
            if(c is Button && (bool) c.Tag) {
               c.Click += OnClick;
               c.Location = new Point(10, i*32+6);
               c.Size = new Size(dialog.Width - 30, 24);
            }
            else {
               Label label = new Label();
               label.Text = key;
               label.Location = new Point(10, i*32+6);
               label.Size = new Size(dialog.Width / 3 - 15, 24);
               dialog.Controls.Add(label);
               c.Location = new Point(dialog.Width / 3 + 10, i*32+4);
               c.Size = new Size(2 * dialog.Width / 3 - 20, 24);
            }
            dialog.Controls.Add(c);
         }
         if(stdButtons != StandardButtons.None) {
            int i = count;
            if((stdButtons & StandardButtons.OK) != 0) {
               Button bOK = new Button();
               bOK.Text = "OK";
               bOK.Size = new Size(70, 24);
               bOK.Location = new Point(dialog.Width - 160, i*32 + 6);
               bOK.Click += OnClick;
               dialog.Controls.Add(bOK);
            }
            if((stdButtons & StandardButtons.Cancel) != 0) {
               Button bCancel = new Button();
               bCancel.Text = "Cancel";
               bCancel.Size = new Size(70, 24);
               bCancel.Location = new Point(dialog.Width - 80, i*32 + 6);
               bCancel.Click += OnClick;
               dialog.Controls.Add(bCancel);
            }
         }
         clickedButton = "";
         dialog.ShowDialog();
         return clickedButton;
      }
      
      void OnClick(object sender, EventArgs e) {
         clickedButton = ((Button) sender).Text;
         dialog.Close();
      }

      public object GetValue (string key) {
         try {
            Type retType = initialValues[key].GetType();
            return  retType.GetMethod("Parse", new Type[]{typeof(string)})
               .Invoke(null, new object[]{keyToControl[key].Text});
         }
         catch(TargetInvocationException) {
            return initialValues[key];
         }
            
      }
      
      public IEnumerable<string> Keys {
         get {
            foreach(string key in keys)
               yield return key;
         }
      }


   }
   
   internal class DBForm : Form {
      internal DBForm() : base() {
         DoubleBuffered = true;
      }
   }
}
