namespace Grendel.Communication {
   using System;
   using System.Collections.Generic;

/// <summary>
/// Rozhraní objektů schopných přijímat zprávy.
/// </summary>    
public interface IMessageReceiver {
   void MessageReceiving(object message);
}

/// <summary>
/// Rozhraní objektů zajišťujících komunikaci mezi objekty aplikace. Komunikující
/// objekty se sdružují do pojmenovaných komunikačních skupin prostřednictvím
/// registrace u komunikátoru. Následně lze posílat zprávy všem objektům ve skupině.
/// KOmunikátor je implementací návrhového vzoru "Pozorovatel" (Observer).
/// </summary>
public interface ICommunicator {
   /// <summary>
   /// zaslání zprávy všem čelnům dané komunikační (cílové) skupiny
   /// </summary>
   int SendMessage(string targetGroup, object message);
   /// <summary>
   /// registrace do nové komunikační skupiny. Prvním parametrem je jméno
   /// skupiny druhým reference na objekt příjemce (tomuto objektu budou doručovány
   /// zprávy, typicky to bývá objekt this). Pokud skupina existuje je objekt
   /// přidán do skupiny, jinak je vytvořena skupina nová. Metoda vrací aktuální
   /// počet objektů ve skupině po provedení registrace
   ///  (pouze informativní údaj, počet se může dále měnit)
   /// </summary>
   void Subscribe(string targetGroup, IMessageReceiver receiver);
   /// <summary>
   /// Odhlášení z komunikační skupiny.
   /// </summary>
   void SignOff(string targetGroup, IMessageReceiver receiver);
   /// <value>
   /// vrací unikátní identifikátor skupiny. Vhodné v situacích, kdy
   /// by měl komunikační skupinu tvoří jediný objekt (posílání zpráv
   /// nahrazuje přímé volání metod daného objektu)
   /// </value>
   string UniqueGroupIdentifier {get;}
}

   /// <summary>
   /// implementace komunikátoru, který podporuje komunikaci uvnitř
   /// aplikace (v jediném procesu na jediném počítači).
   /// </summary>
public class LocalCommunicator : ICommunicator {
      private int uid = 0;
      private Dictionary<string, List<IMessageReceiver>> mapping = 
         new Dictionary<string,List<IMessageReceiver>>();
      private static ICommunicator nexus = null;
      
      
      public int SendMessage (string targetGroup, object message) {
         int count = 0;
         if (mapping.ContainsKey(targetGroup)) 
            foreach(IMessageReceiver receiver in mapping[targetGroup]) {
               receiver.MessageReceiving(message);
               count++;
            }
      return count;
      }
      
      public void Subscribe (string targetGroup, IMessageReceiver receiver) {
         if (mapping.ContainsKey(targetGroup))
           mapping[targetGroup] = new List<IMessageReceiver>();
         mapping[targetGroup].Add(receiver);
      }
      
      public void SignOff (string targetGroup, IMessageReceiver receiver) {
         if(mapping.ContainsKey(targetGroup)) {
            mapping[targetGroup].Remove(receiver);
            if(mapping[targetGroup].Count == 0) 
               mapping.Remove(targetGroup);
         }
      }
         
      public string UniqueGroupIdentifier {
            get { return "#" + uid++; }
      }

        /// <value>
        /// vrací standardní komunikační objekt aplikace (singleton).
        /// Měl by být využíván pro většinu lokální komunikace (výjimkou
        /// jsou pouze případy, kdy je požadován speciální komunikační kanál)
        /// </value>
      public static ICommunicator Nexus {
         get {
            if(nexus == null)
               nexus = new LocalCommunicator();
            return nexus;
         }
      }

   }
   
}