namespace Grendel.Base {
	using System.Collections.Generic;
	using System.Diagnostics;
	
	public class CompositeShape : BaseShape {
		List<IShape> subshapes;
		
		public CompositeShape(Canvas canvas, IShape parentShape) 
			: base(canvas, parentShape) {
			subshapes = new List<IShape>();
		}
		
		public CompositeShape(Canvas canvas) 
			: this(canvas, null) {}
		
		public void AddSubshape(IShape subshape, Position location) {
			if( !subshapes.Contains(subshape) ) {
				Debug.Assert(subshape.ParentShape == null, "Only subshape without parent is appendable to composite object");
				subshapes.Add(subshape);
				subshape.ParentShape = this; //reparent
				subshape.Location = location;
			}
		}
		
		public override void Painting ()
		{
			//nothing is painted
		}

	}
	
	
	public class CompositeInteractiveShape : BaseInteractiveShape {
		List<IShape> subshapes;
		
		public CompositeInteractiveShape(Canvas canvas, IShape parentShape) 
			: base(canvas, parentShape) {
			subshapes = new List<IShape>();
		}
		
		public CompositeInteractiveShape(Canvas canvas) 
			: this(canvas, null) {}
				
		public void AddSubshape(IShape subshape, Position location) {
			if( !subshapes.Contains(subshape) ) {
				Debug.Assert(subshape.ParentShape == null, "Only subshape without parent is appendable to composite object");
				subshapes.Add(subshape);
				subshape.ParentShape = this; //reparent
				subshape.Location = location;
			}
		}
		
		public void AddSubshape(IInteractiveShape subshape, Position location, bool deactivate) {
				AddSubshape(subshape, location);
				if(deactivate)
					subshape.IsActive = false;
					
		}
		
		private Parallelogram Wrapper() {	
			if (subshapes.Count == 0)
				return Parallelogram.FromSimpleRectangle(Position.Origin, Position.Origin);
			
			List<Position> pos = new List<Position>();
			
			foreach(IShape shape in subshapes) {
				if(shape is IInteractiveShape) {
					IInteractiveShape ishape = shape as IInteractiveShape;
					Parallelogram p = Parallelogram.FromSimpleRectangle(ishape.LeftTopCorner,
				                                                       	   ishape.RightBottomCorner);
					WorldMatrix lt = new WorldMatrix(ishape.LocalTransformation); //local transformations must be cloned
					Parallelogram realp = 
						p.Transform(lt.Translate(ishape.Location - Position.Origin));
				
					pos.AddRange(realp.VertexEnumerator());		                    
				}
				else
					pos.Add(shape.Location);
			}
			
			Position l = pos[0];
			Position r = l;
			Position t = l;
			Position b = l;
			
			foreach(Position p in pos) {
				if (p.IsLeftFrom(l))
					l = p;
				else if (p.IsRightFrom(r))
					r = p;
				if (p.IsAbove(t))
					t = p;
				else if (p.IsBelow(b))
					b = p;
			}
			
			return Parallelogram.FromSimpleRectangle(new Position(l.X, t.Y), new Position(r.X, b.Y));
		}
		
		public override void Painting ()
		{
			Parallelogram pg = Wrapper();
			for(int i=1; i<= 4; i++)
				Canvas.DrawLine(this, pg.Vertex(i-1), pg.Vertex(i%4), Color.Gray + Color.White);
				
		}
		
		public override Position LeftTopCorner {
			get {
				Parallelogram pg = Wrapper();
				return pg.Vertex(0);
			}
		}
		
		public override Position RightBottomCorner {
			get {
				Parallelogram pg = Wrapper();
				return pg.Vertex(2);
			}
		}
		
		public override void Interaction (MouseButtons button, Position location)
		{

		}



	}
	
}