// DynamicObject.cs created with MonoDevelop
// User: fiser at 14:42 20.7.2008
//
// To change standard headers go to Edit->Preferences->Coding->Standard Headers
//

namespace Grendel.Base {  
   using System;
   using System.Collections.Generic;
   using System.Diagnostics;
   using Grendel.Extra;

    /// <summary>
    /// Rozhraní dynamických objektů plátna. Dynamické objekty
    /// mohou reagovat na animační (simulační) čas, neboť 
    /// během každého snímku je volána jejich metoda 
    /// <see cref="AnimationStep"/>. 
    /// </summary>
   public interface IDynamicObject : ICanvasObject {
       /// <summary>
       /// Metoda volaná během každého animačního kroku.
       /// Vrací true, pokud změna provedená v metodě
       /// vyžaduje překreslení plátna (tj. v rámci metody
       /// byl změněn alespoň jeden tvar (resp. přesněji
       /// jeho vizualizovaná vlastnost).
       /// </summary>
      bool AnimationStep(int frame);
   }
   
    /// <summary>
    /// základní implementace dynamického objektu (poskytuje jen fukčnost
    /// základního objektu plátna)
    /// </summary>
   public abstract class BaseDynamicObject : BaseCanvasObject, IDynamicObject {
      
      public BaseDynamicObject(Canvas canvas) : base(canvas) {}
      
      public abstract bool AnimationStep(int frame);
   }
   
   /// <summary>
   /// základní implementace animátoru, tj. objektu, který se sám
   /// nezobrazuje, v rámci animace však ovlivňuje přidružený tvar
   /// </summary>
   public abstract class Animator : BaseDynamicObject {
      IShape animatedObject;
      
      public Animator(Canvas canvas, IShape animatedObject) 
         : base(canvas) {
         this.animatedObject = animatedObject;
      }
      
      public IShape AnimatedObject {
         get {
            return animatedObject;
         }
      }
      
   }
   
    /// <summary>
    /// animátor, kerý animuje posun přidruženého tvaru 
    /// z aktuální relativní pozice na novou resp. posun
    /// po celé cestě, definované jednotlivými body. 
    /// </summary>
   public class Shifter : Animator {
      private IShape shiftedObject;
      private Queue<Position> targets;
      private double velocity;
      
      public double Velocity {
         get {
            return velocity;
         }
      }
      
      public IShape ShiftedObject {
         get {
            return shiftedObject;
         }
      }
      
      public Shifter(Canvas canvas, IShape shiftedObject, List<Position> LocationPath, 
                      double velocity) 
               : base(canvas, shiftedObject){
         this.targets = new Queue<Position>(LocationPath);
         this.velocity = velocity;
         this.shiftedObject = shiftedObject;
      }
      
      public Shifter(Canvas canvas, IShape shiftedObject, Position LocationTarget, 
                      double velocity) 
               : base(canvas, shiftedObject){
         this.targets = new Queue<Position>();
         targets.Enqueue(LocationTarget);
         this.velocity = velocity;
         this.shiftedObject = shiftedObject;
      }
      
      public override bool AnimationStep (int frame) {
         if(targets.Count == 0) {
			Canvas.Forget(this);
            return false;
		 }
         double step = Canvas.AnimationInterval * Velocity;
         double distance = targets.Peek().Distance(shiftedObject.Location);
         if(distance < step * 1.5) {
            Position target = targets.Dequeue();
			ShiftedObject.Location = target;
            return true;
         }
         Vector targetshift = targets.Peek() - shiftedObject.Location;
         Vector stepshift = (step / targetshift.Length) * targetshift;
         ShiftedObject.Location += stepshift;
         return true;
      }
   }
   
   /// <summary>
   /// abstraktní třída dočasných objektů tj. objektů
   /// jež po definovaném čase sami zanikají.
   /// </summary>
   abstract class TemporaryShape : BaseShape, IDynamicObject {
      private int startFrame;
      private double lifeTime;
      
      public TemporaryShape(Canvas canvas, double lifeTime) 
      : base(canvas) {
         this.lifeTime = lifeTime;
         this.startFrame = Canvas.Frame;
      }
      
      public bool AnimationStep (int frame) {
         if((frame - startFrame)*Canvas.AnimationInterval > lifeTime) {
            Canvas.Forget(this);
            return true;
         }
         else 
            return false;
      }
   }
   
    /// <summary>
    /// Třída objektů, jež kreslí dočasný vyznačovací obdélník
    /// kolem interaktivních objektů.
    /// </summary>
   class TemporaryMarker : TemporaryShape {
      IInteractiveShape shape;
      
      public TemporaryMarker(Canvas canvas, IInteractiveShape shape, double lifeTime)
         : base(canvas, lifeTime) {
          this.shape = shape;
         Location = shape.AbsolutePosition;
         Layer = int.MaxValue; //marker is in the foreground
      }
      
      public override void Painting () {
         Location = shape.AbsolutePosition;        
         Canvas.DrawRectangle(this, shape.LeftTopCorner, shape.RightBottomCorner, BaseColor);
      }

   }
	
    /// <summary>
    /// Pomocná třída pro vytváření delta-listů plánovaných akcí.
    /// Je využívána třídou <see cref="ActionLauncher"/>
    /// </summary>
    public class DeltaListNode {
        private int frame;
        private Action<int> action;
        private DeltaListNode next = null;
        
        /// <value>
        /// další uzel delta-listu (tj. akce, která bude prováděna jako
        /// další v pořadí)
        /// </value>
        public DeltaListNode Next {
            get {
                return next;
            }
        }
        
        /// <summary>
        /// vytváří nový uzel s časovanou akcí. Čas je udáván 
        /// v počtu snímků od aktuálního okamžiku.
        /// </summary>>
        public DeltaListNode(int startFrame, Action<int> action) {
            this.frame = startFrame;
            this.action = action;
        }
        
        /// <summary>
        /// vkládá uzel do delta-listu. Při počátečním volání
        /// je jako oldnode předá první uzel delta-listu.
        /// Při vkládání je metoda volána rekurzivně. Může
        /// být použita i pro vkládání do prázdného delta-listu.
        /// </summary>
        public static DeltaListNode Insert(DeltaListNode oldNode, DeltaListNode newNode) {
            if(oldNode == null)
                return newNode;
            if(oldNode.frame > newNode.frame) {
                oldNode.frame -= newNode.frame;
                newNode.next = oldNode;
                return newNode;
            }
            else {
                newNode.frame -= oldNode.frame;
                oldNode.next = Insert(oldNode.next, newNode);
                return oldNode;
            }
        }
        
        /// <summary>
        /// Tato metoda musí být volána v rámci obsluhy
        /// každého snímku. Snižuje čekání u všech
        /// plánovaných akcí, a pokud čekání první
        /// (= nejdříve plánované) akce klesne na nulu
        /// vrátí příslušnou akci (jinak vrací null).
        /// V tomto případě musí být metoda bezprostředně
        /// volána na další (vlastnost <see cref="Next"/>) uzel
        /// seznamu (ten totiž může být naplánován na stejný
        /// snímek). To se musí opakovat, dokud je co provádět
        /// (= návratová hodnota je tůzná od null). 
        /// </summary>
        public Action<int> FrameDecrement() {
            frame--;
            return frame == -1 ? action : null;
        }
    }
	/// <summary>
	/// ActionLauncher slouží ke spouštění akcí v předem 
	/// definovaném animačním čase (přesněji od zadaného
	/// animačního snímku). Akcemi jsou delagáti, kterým
	/// je při aktivaci předáno číslo snímku.
	/// </summary>
	public class ActionLauncher : BaseDynamicObject {
		private DeltaListNode deltaList = null;
		
        /// <summary>
        /// vytváří prázdný spouštěč akcí (žádná akce není
        /// naplánována). Akce by měly být následně předány pomocí 
        /// <see cref="AddAction"/> 
        /// </summary>
        public ActionLauncher(Canvas canvas) 
            : base(canvas) {}
        
        /// <summary>
        /// vytváří spouštěč akcí a inicializuje jej ze slovníku.
        /// Klíčem slovníku by měl být plánovaný snímek, hodnotou
        /// příslušná akce. 
        /// </summary>
		public ActionLauncher(Canvas canvas, IDictionary<int, Action<int>> actions) 
            : base(canvas) {
            foreach(int frame in actions.Keys) {
                AddAction(frame, actions[frame]);
            }
		}
        
        
        /// <summary>
        /// přidává delší plánovanou akci. Číslo snímku je relativní
        /// tj. akce bude provedena pro framesAfter snímcích od
        /// okamžiku přidání.
        /// </summary>
        public void AddAction(int framesAfter, Action<int> action) {
            deltaList = DeltaListNode.Insert(deltaList, 
                              new DeltaListNode(framesAfter, action));
        }
        
        public override bool AnimationStep (int frame) {
            while(deltaList != null) {
                Action<int> action = deltaList.FrameDecrement();
                if (action == null)
                    return false;
                action(frame);
                deltaList = deltaList.Next;
            }
            return true;
        }
	}
}