namespace Grendel.Example {
   using System;
   using Grendel.Base;
   using Grendel.Extra;
	
   /// <summary>
   /// interaktivní verze řeckého kříže  (s dialogem vlastností). Ukázkový a testovací tvar.
   /// </summary>
   public class InteractiveGreekCross : GreekCross, IInteractiveShape {
      public InteractiveGreekCross(Canvas canvas, double size) :base(canvas, size){}
		
	  private bool active = true;
      
      public Position LeftTopCorner {
         get { return new Position(-Size/2, - Size/2); }
      }
      
      public Position RightBottomCorner {
         get { return new Position(Size/2, Size/2); }
      }

      public void Interaction (MouseButtons button, Position location) {
         new TemporaryMarker(Canvas, this, 3);
         if(button.IsPressedLeft)
            new Inspector(Canvas, this).PropertyDialog();
         if(button.IsPressedRight)
            new Inspector(Canvas, this).MethodLauncher();
      }
		
	  public bool IsActive {
			get {return active;}
			set {active = value;}
	  }

   }
	
/// <summary>
 /// Pravidelná hvězda se specifikovaným očtem cípů a nastavitelným vnitřním a vnějším 
 /// průměrem. Vztažný bod je ve středu hvězdy. Ukázkový a testovací tvar.
 /// </summary>
public class Star : BaseInteractiveShape, ICanvasCloneable {
   int n;
   private double ip;
   private double p;
   private  bool maltesianEfect;
   private Color outlineColor;
      
   public Star(Canvas canvas, int corners, double radius, 
                 double internalRadius, Color outlineColor) : base(canvas){
         this.n = corners;
         this.p = radius;
         this.ip = internalRadius;
         this.outlineColor = outlineColor;
      }
      
   public Star(Canvas canvas)
      : this(canvas, 8, 80.0, 50.0, Color.Black){}
      
   public ICanvasObject Clone () {
         Star clone = new Star(Canvas, Corners, Radius, InternalRadius, OutlineColor);
         clone.InheritGroupsFrom(this);
         clone.Location = Location;
	     clone.LocalTransformation = LocalTransformation;
         //clone.ParentShape = ParentShape;
         clone.Layer = Layer;
         clone.BaseColor = BaseColor;
         clone.MaltesianEfect = MaltesianEfect;
         return clone;
   }
      
   public void CloneLeft() {
         Clone();
         MoveLeft();
   }
      
      public void MoveLeft() {
         Position newPos = Location + new Vector(2*p, 0);
         new Shifter(Canvas, this, newPos, 33.0);
      }
   
      
   /// <value>
   /// vnější poloměr (vnějších cípů)
   /// </value>   
   public double Radius {
      get {
         return p;
      }
      set {
         p = value;
      }
   }
   
      /// <value>
      /// počet cípů (musí být větší nebo roven 2).
      /// </value>
   public int Corners {
      get {
         return n;
      }
      set {
         n = value;
      }
   }
   
		/// <value>
		/// vnitřní průměr
		/// </value>
   public double InternalRadius {
      get {
         return ip;
      }
      set {
         ip = value;
      }
   }

      /// <value>
	  ///efekt spočívající ve vyplnění jen nekterých sekcí mezi cípy hvězdy (ostatní jsou nevyplněné,
	  ///tj. zcela průhledné)
      /// </value>
   public bool MaltesianEfect {
      get {
         return maltesianEfect;
      }
      set {
         maltesianEfect = value;
      }
   }
      
   public override Position LeftTopCorner {
         get { return new Position(-p, -p); }
   }
      
   public override Position RightBottomCorner {
         get { return new Position(p, p); }
   }

		/// <value>
		/// barva orámování
		/// </value>
   public Color OutlineColor {
      get {
         return outlineColor;
      }
      set {
         outlineColor = value;
      }
   }

   public override void Painting () {
       double krok = 2*Math.PI / n;
       for(int i=0; i<n; i++) {
         double puhel = krok * i;
         double iuhel = puhel + krok/2;
         double kuhel = puhel + krok;
         Position b1 = new Position(p*Math.Cos(puhel), p*Math.Sin(puhel));
         Position b2 = new Position(ip*Math.Cos(iuhel), ip*Math.Sin(iuhel));
         Position b3 = new Position(p*Math.Cos(kuhel), p*Math.Sin(kuhel));
         Canvas.DrawFilledTriangle(this, b1, Position.Origin, b2, BaseColor);
         if (!maltesianEfect)
            Canvas.DrawFilledTriangle(this, b2, Position.Origin, b3, BaseColor);
         Canvas.DrawLine(this, b1, b2, 1, OutlineColor);
         Canvas.DrawLine(this, b2, b3 , 1, OutlineColor);
         }
   }
   
   public override void Interaction (MouseButtons button, Position location) {
         new TemporaryMarker(Canvas, this, 3.0);
         if(button.IsPressedLeft)
            new Inspector(Canvas, this).PropertyDialog();
         if(button.IsPressedRight)
            new Inspector(Canvas, this).MethodLauncher();   
   }

}
	
   /// <summary>
   /// Simple self animated shape displaying current animation frame. It serve primary for testing
   /// purposes (animation speed, problems with repainting)
   /// </summary>
   public class FrameCounter : Text, IDynamicObject {
      public FrameCounter(Canvas canvas) : base(canvas, "") {
         CurrentText = Canvas.Frame.ToString();
      }
      
      public bool IsActive {
         get { return true; }
         set {  }
      }
      
      public bool AnimationStep (int frame) {
         CurrentText = frame.ToString(); 
         return true;
      }

   }
}