// ExtraShapes.cs created with MonoDevelop
// User: fiser at 21:31 25.7.2008
//
// To change standard headers go to Edit->Preferences->Coding->Standard Headers
//

namespace Grendel.Extra {

   using System;
   using Grendel.Base;
   using System.IO;
   using System.Resources;
   using System.Reflection;
	
/// <summary>
/// Společná třída čárových objektů tj. objektů složených jen s čar s danou pevnou šířkou.
/// </summary>	
   public abstract class LinearShape : BaseShape {
		private double lineWidth = 1.0;
		
	public LinearShape(Canvas canvas) : base(canvas) {}
		
	/// <value>
	/// šířka čar (v pixelech). Implicitně je rovna 1.0.
	/// </value>
		public double LineWidth {
		   	get {
		   		return lineWidth;
		   	}
		   	set {
		   		lineWidth = value;
		   	}
		   }   
	}
	
	/// <summary>
	/// Jednotková kružnice se středem ve vztažném bodě.
	/// </summary>
	public class UnitCircle : LinearShape {
		public UnitCircle(Canvas canvas) : base(canvas) {}
		
		public override void Painting () {
             double kappa = 0.5522847498;
			Canvas.DrawBezierCurve(this, new Position(-1.0, 0.0),
                                           new Position(-1.0, -kappa),
                                           new Position(-kappa, -1.0),
                                           new Position(0.0, -1.0), LineWidth, BaseColor);
			Canvas.DrawBezierCurve(this, new Position(0.0, -1.0),
                                           new Position(kappa, -1.0),
                                           new Position(1.0, -kappa),
                                           new Position(1.0, 0.0), LineWidth, BaseColor);
			Canvas.DrawBezierCurve(this, new Position(1.0, 0.0),
                                           new Position(1.0, kappa),
                                           new Position(kappa, 1.0),
                                           new Position(0.0, 1.0), LineWidth, BaseColor);
            Canvas.DrawBezierCurve(this, new Position(0.0, 1.0),
                                           new Position(-kappa, 1.0),
                                           new Position(-1.0, kappa),
                                           new Position(-1.0, 0.0), LineWidth, BaseColor);            
		}
	}
	
	/// <summary>
	/// Jednotková úsečka vycházející z bodu (0,0) a končící v bodě (1,0) tj. vodorovná.
	/// </summary>
	public class UnitHorizontalLine : LinearShape {
		public UnitHorizontalLine(Canvas canvas) : base(canvas) {}
		
		public override void Painting () {
			Canvas.DrawLine(this, Position.Origin, new Position(1.0, 0.0), LineWidth, BaseColor);
		}
	}
	
   /// <summary>
   /// Jednoduchý kříž s rameny stejné délky (= řecký kříž). Vztažný bod je uprostřed v křížení.
   /// </summary>
 public class GreekCross : LinearShape {
      private double size;
      
      
      /// <value>
      /// délka ramen (v pixelech)
      /// </value>
      public double Size {
         get {
            return size;
         }
         set {
            size = value;
         }
      }
      
      public GreekCross(Canvas canvas, double size) : base(canvas) {
         this.size = size;
      }
        
      
      public override void Painting () {
         Canvas.DrawLine(this, new Position(-size/2,0), new Position(size/2, 0), LineWidth, BaseColor);
         Canvas.DrawLine(this, new Position(0, -size/2), new Position(0, size/2), LineWidth,  BaseColor);       
      }

   }
   
   /// <summary>
   /// Zobrazení bitmapy z grafického soubouru.
   /// </summary>
   public class StaticBitmap : BaseShape {
      private string filename;
      private IBitmap bitmap = null;
      
      public StaticBitmap(Canvas canvas, string fileName) : base(canvas) {
         FileName = fileName;
      }
      
      /// <value>
      /// Jméno grafického souboru s bitmapou. Podporované formáty závisí na použité kreslící knihovně,
      /// běžné formáty (PNG, JPG) by však měly být podporovány.
      /// </value>d
      public string FileName {
         get {
            return filename;
         }
         set {
            filename = value;
            if(bitmap != null)
               bitmap.Dispose();
            Stream s = File.OpenRead(value);
            bitmap = Canvas.LoadBitmap(s);
         }
      }
      
      public override void Painting () {
         Canvas.DrawBitmap(this, Position.Origin, bitmap);
      }      
   }
   
/// <summary>
/// Zobrazení bitmapy ze zdrojů připojených k aplikaci (tj. například k assembly).
/// </summary>	
   public class ResourceBitmap : BaseShape {
      IBitmap b = null;
      
      public ResourceBitmap(Canvas canvas, string resourceID) : base(canvas) {
         Assembly assembly = Assembly.GetAssembly(typeof(Canvas));
         Stream s = assembly.GetManifestResourceStream(resourceID);
         b = Canvas.LoadBitmap(s);
      }
      
      public override void Painting () {
          Canvas.DrawBitmap(this, Position.Origin, b);        
      }

   }
   
  /// <summary>
  /// Zobrazení jednoduchého textu v standardním strojopisném fontu (je možno změnit pouze barvu).
  /// Vztažný bod je v levém horním rohu.
  /// </summary> 
   public class Text : BaseShape {
      private string text;
      
      
      /// <value>
      /// zobrazovaný text 
      /// </value>
      public string CurrentText {
         get {
            return text;
         }
         set {
            text = value;
         }
      }
      
      public Text(Canvas canvas, string text) : base(canvas){
         this.text = text;
      }
      
      public override void Painting () {
         Canvas.DrawText(this, Position.Origin, CurrentText, Font.Monospace, BaseColor);
      }
   }
   
   /// <summary>
  /// Zobrazení jednoduchého textu ve zvoleném fontu.
  /// Vztažný bod je v levém horním rohu.
   /// </summary>
   public class TextWithFont : Text {
      private Font font;
      
      public TextWithFont(Canvas canvas, string text, Font font) 
      : base(canvas, text){
         this.Font = font;
      }
      
      /// <value>
      /// řez, v němž je text zobrazen
      /// </value>
      public Font Font {
         get {
            return font;
         }
         set {
            font = value;
         }
      }
      
      public override void Painting () {
         Canvas.DrawText(this, Position.Origin, CurrentText, Font, BaseColor);
      }
   }
   
/// <summary>
/// Zobrazení beziérovy křivky (určené čtyřmi body nebo dvěma body a dvěma vektory).
/// </summary>	
public class BezierCurve : LinearShape {
	private Position start;
	private Vector startVector;
	private Position finish;
	private Vector finishVector;
		
    /// <summary>
    /// beziérova křivka popsaná dvěma body a dvěma směrovými vektory
    /// </summary>
    /// <param name="startPoint">
    /// Počáteční bod (P0), z něhož křivka vychází.
    /// </param>
    /// <param name="startVector">
    /// Vektor určující směr křivky z výchozího bodu (P1 - P0)
    /// </param>
    /// <param name="finishPoint">
    /// Koncový bod křivky (P3)
    /// </param>
    /// <param name="finishVector">
    /// Vektor určující směr vstupu křivky do koncového bodu (P3 - P2)
    /// </param>
	public BezierCurve(Canvas canvas, Position startPoint, Vector startVector,
		                    Position finishPoint, Vector finishVector) 
		: base(canvas) {
			this.start = startPoint;
			this.startVector = startVector;
			this.finish = finishPoint;
			this.finishVector = finishVector;
	}
   /// <summary>
   /// Vytvoří beziérovu křivku určenou čtyřmo řídícími body.
   /// </summary>
    public BezierCurve(Canvas canvas, Position p0, Position p1, Position p2,
                            Position p3) : base(canvas) {
            this.start = p0;
            this.startVector = p1 - p0;
            this.finish = p3;
            this.finishVector = p3 - p2;
    }
		
	public override void Painting ()
	{
		Position p0 = start;
		Position p1 = start + startVector;
		Position p2 = finish + (-1.0 * finishVector);
		Position p3 = finish;
		Canvas.DrawBezierCurve(this, p0, p1, p2, p3, LineWidth, BaseColor);
	}

}
   
}