namespace Grendel.Base {  
   using System;
   using System.Diagnostics;
   using Grendel.Communication;
   using System.Collections.Generic;
   
   /// <summary>
   /// základní rozhraní všech tvarů
   /// </summary>
   public interface IShape : ICanvasObject {
      /// <value>
      /// umístění vztažného bodu objektu. Umístění je u podtvarů  
      /// relativní ke vztažnému bodu nadřízeného podtvaru.
      /// U tvarů bez nadřízeného tvaru (= kořenových tvarů)
      /// je umístění vztaženo k fyzickým souřadnicím 
      /// plátna (tj. je absolutní)
      /// </value>
      Position Location {get; set;}
      /// <value>
      /// rodičovský tj. nadřízený tvar. Podřízený tvar je umístěn 
      /// relativně vůči nadřízenému a sdílí jeho transformace (např.
      /// je posouván spolu s rodičem). Závisí na něm také existenčně
      /// tj. při zániku nadřízeného objektu je destruován.
      /// U kořenových objekty tj. objektu, které nejsou součástí žádného
      /// objektu, má tato vlastnost hodnotu null.
      /// </value>
      IShape ParentShape {get; set;}
      /// <value>
      /// absolutní pozice tvaru vůči plátnu. Je vypočítána z umístění
      /// aplikací transformací v řetězci nadřízených objektů.
      /// </value>
      Position AbsolutePosition {get;} 
      /// <value>
      /// základní barva tvaru (typicky je převažující barva např.
      /// barva výplně u jednobarevných objektů). Pokud tvar nemá
      /// základní barvu je možno vrátit tzv. prázdnou barvu (=
      /// plně průhledná černá).
      /// </value>
      Color BaseColor {get; set;}
      /// <value>
      /// vrstva, v níž je tvar kreslen. Tvary ve vyšších vstvách
      /// (= s numericky vyšší hodnotou) zakrývají objekty ve vrstvách
      /// nižších. Základní vrstva je vrstva 0. Lze používat i vrstvy
      /// se zápornou hodnotou.
      /// </value>
      int Layer{get; set;}
	  /// <value>
	  /// lokální (vnitřní) transformace tvaru. Ovlivňuje jak elementární
	  ///  kreslící operace (primitiva tj. Canvas.DrawXXX), tak případné
	  /// podřízené objekty.
	  /// </value>
	  WorldMatrix LocalTransformation {get; set;}
      /// <value>
      /// globální (vnější) transformace objektu. Transformace objektu
      /// jako celku, zděděné z řetězce nadřízených objektů. 
      /// </value>
	  WorldMatrix TotalTransformation {get;}
      /// <summary>
      /// hlavní kreslící metoda. Musí obsahovat všechny elemntární 
      /// příkazy (primitiva) pro vykreslení tvaru. Kreslící metoda je volána
      /// volána při inicializace plátna, při jeho obnovování (např. po zakrytí
      /// okna, resp. v jednotlivých animačních krocích.
      /// Tato metoda musí být předefinována v odvozených třídách,
      /// pokud je tvar vytvářen jako elementární (tj. nesložený z jiných tvarů)
      /// </summary>        
      void Painting();
   }  
   
   
   /// <summary>
   /// ozhraní pro interaktivní tvary tj. tvary které jsou schopny reagovat na vstupní událostiu
   /// (v současnosti jsou pouze události myši tj. stisk tlačítka myši nad daným tvarem)
   /// </summary>
   public interface IInteractiveShape : IShape {
      /// <value>
      /// Levý horní roh obdélníkové oblasti, která by měla obsahovat všechny viditelné části tvaru.
      /// Tato oblast definuje aktivní část tvaru tj. oblast v níž jsou zachytávány události myši
      /// a následně doručovány tomuto interaktivnímu tvaru. Pozice je relativní vůči vztažnému
      /// bodu daného tvaru (vztažný bod by neměl být umístěn vně aktivní oblasti) 
      /// </value>
      Position LeftTopCorner {get;}
      /// <value>
      /// Pravý dolní roh obdélníkové oblasti, která by měla obsahovat všechny viditelné části tvaru.
      /// Tato oblast definuje aktivní část tvaru tj. oblast v níž jsou zachytávány události myši
      /// a následně doručovány tomuto interaktivnímu tvaru. Pozice je relativní vůči vztažnému
      /// bodu daného tvaru (vztažný bod by neměl být umístěn vně aktivní oblasti) 
      /// </value>
      Position RightBottomCorner {get;}
      /// <summary>
      /// obsluha vstupních (myších) událostí. Kód obsahuje reakci tvaru na stisk tlačítka myši
      /// uvnitř aktivní oblasti.
      ///</summary> 
      void Interaction(MouseButtons button, Position location); 
	  /// <value>
	  /// interaktivní objekt je běžně aktivní, tj. reaguje na vstupní události. Pomocí vlastnosti IsActive
	  /// je však možno tvar dočasně deaktivovat. To je užitečné například ve složených objektech.
	  /// </value>	
	  bool IsActive {get; set;}
   }
   
   /// <summary>
   /// Základní implementace objektů plátna, jenž zahrnuje registraci a základní nastavení komunikace
   /// pomocí komunikačních skupin. Doporučená základní třída všech objektů plátna (nicméně stále
   /// je možná nezávislá přímá implementace rozhraní) 
   /// </summary>
   public abstract class BaseCanvasObject : ICanvasObject {
    private Canvas canvas;
    private IList<string> comGroups;
    
      /// <summary>
      /// Základní konstruktor, který zajišťuje registraci a základní správu komunikačních skupin.
      /// Jediným parametrem je plátno, jehož součástí se objekt stane.
      /// </summary>
    public BaseCanvasObject(Canvas canvas) {
         this.canvas = canvas;
         canvas.Remember(this);
         comGroups = new List<string>();
    }
    
        /// <value>
        /// vrací plátno, k němuž objekt patří
        /// </value>
    public Canvas Canvas {
         get { return canvas; }
    }

    public IList<string> ComunnicationGroups {
       get {
          return comGroups;
       }
    }
      
    public void AddToCommunicationGroup (string groupName) {
         Debug.Assert(!comGroups.Contains(groupName), "Group name is not unique");
         comGroups.Add(groupName);
         LocalCommunicator.Nexus.Subscribe(groupName, this);
    }
  
    public void RemoveFromCommunicationGroup (string groupName) {
         Debug.Assert(comGroups.Contains(groupName), "Removed group name is not present"); 
         comGroups.Remove(groupName);
         LocalCommunicator.Nexus.SignOff(groupName, this);
    }
      
    public virtual void MessageReceiving(object message) {
         // empty body (must be overrided)
    }

    protected void InheritGroupsFrom (ICanvasObject templet) {
         foreach(string group in templet.ComunnicationGroups)
            comGroups.Add(group);
    }
      
    public override string ToString () {
         return base.ToString() + " on " + canvas.ToString();
    }
   }
   
   /// <summary>
   /// Zaákladní implemntace tvaru. Implementace zajišťuje základní podporu (umístění, vrstvy)
   /// ale vlastní kreslení objektu je nedefinováno (musí být definováno v odvozených třídách)
   /// </summary>
   public abstract class BaseShape : BaseCanvasObject, IShape {
      private Position location;
      private IShape parent;
      private Color baseColor;
	  private WorldMatrix matrix;
      private int layer;
      
      /// <summary>
      /// Základní (nejobecnější) konstruktor tvaru, v němž je nutno uvést plátno a nadřízený (rodičovský) tvar.
      ///</summary>
      public BaseShape(Canvas canvas, IShape parentShape) 
       : base(canvas) {
         this.parent = parentShape;
         Location = Position.Origin;
         BaseColor = Color.Black;
		 LocalTransformation = new WorldMatrix();
         layer = 0;
      }
      
        /// <summary>
        /// Konstruktor kořenových tvarů (tj. tvarů nezahrnutých do nadřízeného tvaru)
        /// </summary>
      public BaseShape(Canvas canvas) : this(canvas, null){}
      
      public Position Location {
         get {return location;}
         set {location = value;}
      }
      
      public IShape ParentShape {
         get { return parent; }
		 set {parent = value;}
      }
         
      public Position AbsolutePosition {
            get {
				return TotalTransformation.Transform(Position.Origin);
            }
      } 

      public Color BaseColor {
         get {
            return baseColor;
         }
         set {
            baseColor = value;
         }
      }
      
      public int Layer {
         get { return layer; }
         set { 
            if(value != layer) {
               Canvas.MoveToLayer(this, layer, value);
               layer = value;   
            }
         }
      }

	  public WorldMatrix LocalTransformation {
			get {return matrix;}
			set {matrix = value;}
	  }
		
	  public WorldMatrix TotalTransformation {
			get {
				WorldMatrix tm = new WorldMatrix(LocalTransformation);
				tm.Translate(Location - Position.Origin);
				if(ParentShape == null)
					return tm;
				else
					return tm.AddTransformations(ParentShape.TotalTransformation);					
			}
	  }
		
      public abstract void Painting();
   }
	
    /// <summary>
    /// Základní implementace interaktivních tvarů. Kromě zděděné podpory (registrace,
    /// umístění) je přidána pouze podpora uloženého příznaku aktivity (neaktivní
    /// objekt nereeaguje na vstupní události). Vlastní kreslení a reakce musí
    /// být dodefinována v podřízených třídách.
    /// </summary>
	public abstract class BaseInteractiveShape : BaseShape, IInteractiveShape {
		private bool active;
		
		public BaseInteractiveShape(Canvas canvas, IShape parentShape) 
       		: base(canvas, parentShape) {
				active = true;
      }
      
      public BaseInteractiveShape(Canvas canvas) : this(canvas, null){}
		
		public bool IsActive {
			get {return active;}
			set {active = value;}
		}
		
		public abstract Position LeftTopCorner {get;}
		public abstract Position RightBottomCorner {get;}
		
		public abstract void Interaction(MouseButtons button, Position location);
	}
}
